#include <iostream>
#include <vector>
#include <algorithm>
using namespace std;
using ll = long long;

struct point {
    ll x, y;

    point() {}
    
    point(ll x_, ll y_): x(x_), y(y_) {}

    bool operator<(point pt) const {
        return x < pt.x || (x == pt.x && y < pt.y);
    }

    friend ostream& operator<<(ostream& o, point pt) {
        o << "(" << pt.x << ' ' << pt.y << ")";
        return o;
    }
};

bool cmpy(point p1, point p2) {
    return p1.y < p2.y || (p1.y == p2.y && p1.x < p2.x);
}

const int MAXN = 1e5 + 123;
point pt[MAXN];

ll solve_1(int n, ll w, ll h) {
    sort(pt, pt + n);
    ll len = 0;
    ll px = 0;
    ll ans = 0;
    for (int i = 0; i < n; ++i) {
        ans += (pt[i].x - px) * len;
        len = max(len, h - pt[i].y);
        px = pt[i].x;
    }
    return ans + (w - px) * len;
}

struct rectangle {
    point ld;
    point ru;

    rectangle() {}

    rectangle(point ld_, point ru_): ld(ld_), ru(ru_) {}

    rectangle(pair<point, int> lig, ll w, ll h) {
        if (lig.second == 1) {
            ld = lig.first;
            ru = {w, h};
        }
        if (lig.second == 2) {
            ld = {0, lig.first.y};
            ru = {lig.first.x, h};
        }
        if (lig.second == 3) {
            ld = {0, 0};
            ru = lig.first;
        }
        if (lig.second == 4) {
            ld = {lig.first.x, 0};
            ru = {w, lig.first.y};
        }
    }

    ll sqr() {
        return max(ru.x - ld.x, 0ll) * max(ru.y - ld.y, 0ll);
    }

    rectangle intersect(rectangle r) {
        return {{max(ld.x, r.ld.x), max(ld.y, r.ld.y)}, {min(ru.x, r.ru.x), min(ru.y, r.ru.y)}};
    }

    friend ostream& operator<<(ostream& o, rectangle r) {
        o << "{" << r.ld << ' ' << r.ru << "}";
        return o;
    }
};


ll solve_12_part(int n, ll w, ll h) {
    sort(pt, pt + n, cmpy);
    ll py = 0;
    ll plen = 0;
    ll psqr = 0;
    ll ans = 0;
    for (int i = 0; i < n; ++i) {
        psqr += (pt[i].y - py) * plen;
        if (i)
            ans = max(ans, psqr + w * (h - pt[i].y));
        plen = max(plen, pt[i].x);
        py = pt[i].y;
    }
    return max(ans, solve_1(n, w, h));
}

ll solve_12(int n, ll w, ll h) {
    ll ans = solve_12_part(n, w, h);
    for (int i = 0; i < n; ++i)
        pt[i] = {w - pt[i].x, pt[i].y};
    return max(ans, solve_12_part(n, w, h));
}

ll solve_1234(int n, ll w, ll h) {
    if (n >= 4) {
        return w * h;
    }
    ll ans = 0;
    for (int aa = 0; aa < (1 << (2 * n)); ++aa) {
        int hoho = aa;
        vector<rectangle> rct(n);
        for (int i = 0; i < n; ++i) {
            int dir = 1 + hoho % 4;
            // cout << dir << ' ';
            rct[i] = rectangle({pt[i], dir}, w, h);
            hoho >>= 2;
        }
        ll cans = 0;
        for (int i = 0; i < n; ++i) {
            cans += rct[i].sqr();
            for (int j = i + 1; j < n; ++j) {
                rectangle rrr = rct[i].intersect(rct[j]);
                cans -= rrr.sqr();
                for (int k = j + 1; k < n; ++k) {
                    cans += rrr.intersect(rct[k]).sqr();
                }
            }
        }
        // for (int i = 0; i < n; ++i) {
        //     cout << rct[i] << endl;;
        // }
        // cout << cans << endl;
        ans = max(ans, cans);
    }
    return ans;
}

signed main() {
    int k;
    cin >> k;
    vector<int> v(k);
    for (int i = 0; i < k; ++i)
        cin >> v[i];
    int t;
    cin >> t;
    while (t--) {
        int n;
        ll w, h;
        cin >> n >> w >> h;
        for (int i = 0; i < n; ++i)
            cin >> pt[i].x >> pt[i].y;
        if (k == 1) {
            cout << solve_1(n, w, h) << '\n';
        } else if (k == 4) {
            cout << solve_1234(n, w, h) << '\n';
        } else if (k == 2 && (v[0] == 1 && v[1] == 2 || v[0] == 2 && v[1] == 1)) {
            cout << solve_12(n, w, h) << '\n';
        }
    }

}
