#include <iostream>
#include <vector>
#include <algorithm>
#include <set>
#include <map>
#include <string>
#include <iomanip>
#include <assert.h>
#include <cmath>

using namespace std;

using ll = long long;
using ld = long double;

#define all(x) begin(x), end(x)
#define rall(x) rbegin(x), rend(x)

const int mod = 1e9 + 7;
const int N = 5010;

struct STree
{
	vector<ll> t;
	vector<ll> m;

	void build(int v, int tl, int tr, vector<ll>& a)
	{
		if (tl == tr)
		{
			t[v] = a[tl];
		}
		else
		{
			int tm = (tl + tr) / 2;

			build(v * 2, tl, tm, a);
			build(v * 2 + 1, tm + 1, tr, a);

			t[v] = min(t[v * 2], t[v * 2 + 1]);
		}
	}

	void push(int v)
	{
		m[v * 2] += m[v];
		m[v * 2 + 1] += m[v];
	
		t[v * 2] += m[v];
		t[v * 2 + 1] += m[v];

		m[v] = 0;
		t[v] = min(t[v * 2], t[v * 2 + 1]);
	}

	void upd(int v, int tl, int tr, int ql, int qr, ll x)
	{
		if (ql > qr) return;
		if (ql == qr && tl == qr)
		{
			t[v] += x;
			m[v] = x;
		}
		else
		{
			push(v);

			int tm = (tl + tr) / 2;

			upd(v * 2, tl, tm, ql, min(qr, tm), x);
			upd(v * 2 + 1, tm + 1, tr, max(tm + 1, ql), qr, x);

			t[v] = min(t[v * 2], t[v * 2 + 1]);
		}
	}

	ll get(int v, int tl, int tr, int ql, int qr)
	{
		if (ql > qr) return 1e10;
		if (tl == ql && tr == qr) return t[v];
		int tm = (tl + tr) / 2;
		ll left = get(v * 2, tl, tm, ql, min(qr, tm));
		ll right = get(v * 2 + 1, tm + 1, tr, max(tm + 1, ql), qr);
		return min(left, right);
	}
};

vector<int> g[N];
ll c[N];
int cnt[N];
int p[N];
int l[N], r[N];
int up[N], h[N];
int sz[N], m;
int tin[N], tout[N];
ll dp[N], cost[N];
vector<ll> et;
set<pair<ll, int>> s[N];
STree t;

int tim = 0;

void dfs_sz(int v, int p = -1)
{
	h[v] = p == -1 ? 0 : h[p] + 1;
	sz[v] = 1;
	for (auto& u : g[v])
		if (u != p)
			dfs_sz(u, v), sz[v] += sz[u];

	for (int i = 1; i < g[v].size(); i++)
		if (sz[g[v][i]] > sz[g[v][0]])
			swap(g[v][0], g[v][i]);
}

void dfs0(int v, int p = -1)
{
	tin[v] = et.size();
	et.push_back(r[v]);
	for (int i = 0; i < g[v].size(); i++)
	{
		if (g[v][i] != p)
		{
			dfs0(g[v][i], v);
			if (i == 0)
			{
				up[g[v][i]] = up[v];
			}
			else
			{
				up[g[v][i]] = g[v][i];
			}
		}
	}
	tout[v] = (int)et.size() - 1;
}

ll dfs1(int v, int u)
{
	ll ans = 1e10;
	while (up[v] != up[u])
	{
		assert(v != 0);
		ans = min(ans, t.get(1, 0, m - 1, tin[up[v]], tin[v]));
		v = p[up[v]];
	}

	int fr = min(tin[u], tin[v]);
	int to = max(tin[u], tin[v]);

	ans = min(ans, t.get(1, 0, m - 1, fr, to));

	return ans;
}

void dfs2(int v, int u, ll x)
{
	while (up[v] != up[u])
	{
		assert(v != 0);
		t.upd(1, 0, m - 1, tin[up[v]], tin[v], -x);
		v = p[up[v]];
	}

	int fr = min(tin[u], tin[v]);
	int to = max(tin[u], tin[v]);

	t.upd(1, 0, m - 1, fr, to, -x);
}

void dfs(int v, int p = -1)
{
	for (auto& u : g[v])
	{
		if (u != p)
		{
			dfs(u, v);
			dp[v] += dp[u];
			cost[v] += cost[u];

			if (s[v].size() < s[u].size())
				s[v].swap(s[u]);

			for (auto& x : s[u])
				s[v].insert(x);

			s[u].clear();
		}
	}

	s[v].insert({ c[v], v });

	//cout << v << '\n';

	while (dp[v] < l[v])
	{
		auto [co, u] = *(s[v].begin());
		auto can = dfs1(u, v);
		if (can <= 0)
		{
			s[v].erase({ co, u });
		}
		else
		{
			can = min(can, l[v] - dp[v]);
			dfs2(u, v, can);
			cost[v] += can * co;
			cnt[u] += can;
			dp[v] += can;
		}
	}

}

void solve()
{
	int n;
	cin >> n;

	for (int i = 1; i < n; i++)
	{
		cin >> p[i];
		p[i]--;
		g[p[i]].push_back(i);
		g[i].push_back(p[i]);
	}
	
	for (int i = 0; i < n; i++)
	{
		cin >> c[i];
	}

	for (int i = 0; i < n; i++)
	{
		cin >> l[i] >> r[i];
		dp[i] = 0;
		cnt[i] = 0;
		cost[i] = 0;
		up[i] = i;
		sz[i] = 0;
		h[i] = 0;
		tin[i] = tout[i] = 0;
	}
	tim = 0;
	et.clear();

	dfs_sz(0);
	dfs0(0);
	
	m = et.size();
	t.t.assign(4 * m, 0);
	t.m.assign(4 * m, 0);

	t.build(1, 0, m - 1, et);

	dfs(0);
	s[0].clear();

	for (int v = 0; v < n; v++)
	{
		g[v].clear();
	}

	for (int v = 0; v < n; v++)
	{
		if (dp[v] < l[v] || dp[v] > r[v])
		{
			cout << "-1\n";
			return;
		}
	}

	cout << cost[0] << '\n';
	for (int v = 0; v < n; v++)
	{
		cout << cnt[v] << " ";
	}
	cout << '\n';
}

signed main()
{
	ios_base::sync_with_stdio(0);
	cin.tie(0);

	int t = 1;
	cin >> t;

	while (t--)
	{
		solve();
	}

	return 0;
}
