#include <iostream>
#include <vector>
#include <algorithm>
using ll = long long;
using namespace std;

struct Point {
	int r = 0, c = 0, i = 0;
	Point(int r1, int c1, int i1) : r(r1), c(c1), i(i1) {}
};
bool pointcomp(Point& p1, Point& p2) {
	return p1.r < p2.r;
}

const pair<int, int> INF = { -1000000000, -1 };

struct SegTree {
	int size;
	vector<pair<int,int>> tree;
	vector<int> push;
	SegTree(vector<int>& a) {
		int n = a.size();
		size = 1;
		while (size < n) size *= 2;
		tree.assign(2 * size - 1, INF);
		push.assign(2 * size - 1, 0);
		for (int i = 0; i < n; ++i) tree[size - 1 + i] = { a[i],i };
		for (int i = size - 2; i >= 0; --i) tree[i] = max(tree[2 * i + 1], tree[2 * i + 2]);
	}
	void push_down(int v) {
		//cout << "push_down" << v << "\n";
		tree[v].first += push[v];
		if (v < size - 1) {
			push[2 * v + 1] += push[v];
			push[2 * v + 2] += push[v];
		}
		push[v] = 0;
		//cout << "pushend\n";
	}
	pair<int, int> getmax(int l, int r, int x, int lx, int rx) {
		push_down(x);
		if (l <= lx && rx <= r) return tree[x];
		if (rx <= l || r <= lx) return INF;
		int mid = (lx + rx) / 2;
		return max(getmax(l, r, 2 * x + 1, lx, mid), getmax(l, r, 2 * x + 2, mid, rx));
	}
	pair<int, int> getmax(int l, int r) { return getmax(l, r, 0, 0, size); }
	void inc(int l, int r, int x, int lx, int rx) {
		if (l <= lx && rx <= r) {
			++push[x];
			push_down(x);
			return;
		}
		push_down(x);
		if (rx <= l || r <= lx) return;
		int mid = (lx + rx) / 2;
		inc(l, r, 2 * x + 1, lx, mid);
		inc(l, r, 2 * x + 2, mid, rx);
		tree[x] = max(tree[2 * x + 1], tree[2 * x + 2]);
	}
	void inc(int l, int r) {
		//cout << "inc on " << l << " " << r << "\n";
		inc(l, r, 0, 0, size);
	}
	void set(int v, int x) {
		//cout << "Setg " << v << " " << x << "\n";
		v += size - 1;
		vector<int> stack{ v };
		v = (v + 1) / 2 - 1;
		while (v >= 0) {
			stack.emplace_back(v);
			//cout << "Adding " << v << "\n";
			v = (v + 1) / 2 - 1;
		}
		for (int i = stack.size() - 1; i >= 0; --i) push_down(stack[i]);
		tree[stack[0]] = { x, stack[0] - size + 1 };
		for (int i = 1; i < stack.size(); ++i) {
			push_down(2 * stack[i] + 1);
			push_down(2 * stack[i] + 2);
			tree[stack[i]] = max(tree[2 * stack[i] + 1], tree[2 * stack[i] + 2]);
		}
	}
	int remove(int v) {
		v += size - 1;
		vector<int> stack{ v };
		v = (v + 1) / 2 - 1;
		while (v >= 0) {
			stack.emplace_back(v);
			v = (v + 1) / 2 - 1;
		}
		for (int i = stack.size() - 1; i >= 0; --i) push_down(stack[i]);
		return tree[stack[0]].first;
	}
};

struct LeftLess {
	int size;
	vector<int> tree;
	LeftLess(vector<int>& a) {
		int n = a.size();
		size = 1;
		while (size < n) size *= 2;
		tree.assign(2 * size - 1, -INF.first);
		for (int i = 0; i < n; ++i) tree[size - 1 + i] = a[i];
		for (int i = size - 2; i >= 0; --i) tree[i] = min(tree[2 * i + 1], tree[2 * i + 2]);
	}
	int get(int l, int r, int v, int x, int lx, int rx) {
		if (rx <= l || r <= lx) return size;
		if (l <= lx && rx <= r) {
			if (tree[x] > v) return size;
			while (x < size - 1) {
				if (tree[2 * x + 1] <= v) x = 2 * x + 1;
				else x = 2 * x + 2;
			}
			return x - size + 1;
		}
		int mid = (lx + rx) / 2, le = get(l, r, v, 2 * x + 1, lx, mid);
		if(le == size) return get(l, r, v, 2 * x + 2, mid, rx);
		return le;
	}
	int segmin(int l, int r, int x, int lx, int rx) {
		if (rx <= l || r <= lx) return -INF.first;
		if (l <= lx && rx <= r) {
			return tree[x];
		}
		int mid = (lx + rx) / 2;
		return min(segmin(l, r, 2 * x + 1, lx, mid), segmin(l, r, 2 * x + 2, mid, rx));
	}
};

int main() {
	int n;
	cin >> n;
	vector<int> a(n);
	for (int i = 0; i < n; ++i) cin >> a[i];
	int q;
	cin >> q;
	vector<Point> points(2 * q, Point(0,0,0)), orig;
	for (int i = 0; i < 2*q; ++i) {
		cin >> points[i].r >> points[i].c;
		points[i].c = points[i].r - points[i].c;
		points[i].i = i;
	}
	if (n <= 3000 && q <= 3000) {
		for (int i = 0; i < q; ++i) {
			Point& p1 = points[2 * i];
			Point& p2 = points[2 * i + 1];
			int l1 = 0;
			while (p1.c != 0) {
				p1.c -= (a[l1] <= p1.r);
				++l1;
			}
			int l2 = 0;
			while (p2.c != 0) {
				p2.c -= (a[l2] <= p2.r);
				++l2;
			}
			if (l1 == l2) {
				if (p1.r < p2.r) {
					cout << p1.r << " " << p1.r - p1.c << "\n";
				}
				else {
					cout << p2.r << " " << p2.r - p2.c << "\n";
				}
				continue;
			}
			int minr = n;
			for (int i = min(l1, l2); i < max(l2, l1); ++i) {
				minr = min(minr, a[i]);
			}
			minr -= 1;
			cout << minr << " ";
			int c = 0;
			for (int i = 0; i < min(l1, l2); ++i) c += (a[i] <= minr);
			cout << minr - c << "\n";
		}
		return 0;
	}
	orig = points;
	sort(points.begin(), points.end(), pointcomp);
	vector<int> vals(2 * q), lefts(2*q);
	for (int i = 0; i < 2 * q; ++i) vals[i] = -points[i].c;

	//for (int& el : vals) cout << el << " ";
	//cout << "_\n";

	SegTree tr(vals);
	int left = 0, right = 0, mid = 0;
	pair<int, int> cur;
	cur = tr.getmax(0, 2 * q);
	while (cur.first == 0) {
		//cout << cur.second << "_\n";
		lefts[cur.second] = 0;
		tr.set(cur.second, INF.first);
		//cout << "bef\n";
		cur = tr.getmax(0, 2 * q);
	}
	for (int i = 0; i < n; ++i) {
		left = -1;
		right = 2 * q;
		while (right - left != 1) {
			mid = (left + right) / 2;
			if (points[mid].r < a[i]) left = mid;
			else right = mid;
		}
		//cout << right << ":\n";
		tr.inc(right, 2 * q);
		//for (auto& el : tr.tree) cout << el.first << " ";
		//cout << "\n";
		cur = tr.getmax(right, 2 * q);
		while (cur.first == 0) {
			//cout << cur.second << "__\n";
			lefts[cur.second] = i + 1;
			tr.set(cur.second, INF.first);
			cur = tr.getmax(right, 2 * q);
		}
	}

	vector<int> ls(q, n), rs(q, -1);
	//eftLess rl(a);
	for (int i = 0; i < 2 * q; ++i) {
		ls[points[i].i / 2] = min(ls[points[i].i / 2], lefts[i]);
		rs[points[i].i / 2] = max(rs[points[i].i / 2], lefts[i]);
	}

	//for (int i = 0; i < q; ++i) {
		//cout << ls[i] << " - " << rs[i] << "\n";
	//}
	LeftLess mins(a);
	vector<pair<int, int>> evs;
	vector<int> ansr(q + 1), ansc(q + 1);
	for (int i = 0; i < q; ++i) {
		if (ls[i] == rs[i]) {
			if (orig[2 * i].r < orig[2 * i + 1].r) {
				ansr[i] = orig[2 * i].r;
				ansc[i] = orig[2 * i].c;
			}
			else {
				ansr[i] = orig[2 * i + 1].r;
				ansc[i] = orig[2 * i + 1].c;
			}
		}
		else {
			ansr[i] = mins.segmin(ls[i], rs[i], 0, 0, mins.size) - 1;
			evs.emplace_back(pair<int, int>(ansr[i], i));
		}
	}
	sort(evs.begin(), evs.end());
	vals.clear();
	for (auto& el : evs) vals.emplace_back(0);
	SegTree tre(vals);
	vector<vector<int>> inv(n + 1, vector<int>());
	for (int i = 0; i < evs.size(); ++i) {
		inv[ls[evs[i].second]].emplace_back(i);
	}
	a.emplace_back(0);
	for (int i = 0; i <= n; ++i) {
		for (int& j : inv[i]) {
			ansc[evs[j].second] = tre.remove(j);
		}
		left = -1;
		right = evs.size();
		//cout << "La\n";
		while (right - left != 1) {
			mid = (left + right) / 2;
			//cout << mid << " ";
			if (evs[mid].first < a[i]) left = mid;
			else right = mid;
		}
		//cout << "Lu\n";
		tre.inc(right, tre.size);
	}

	for (int i = 0; i < q; ++i) {
		cout << ansr[i] << " " << ansr[i] - ansc[i] << "\n";
	}
	return 0;
}