#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;
using ll = long long;
using vi = vector<int>;
using vvi = vector<vi>;
using vl = vector<ll>;
using vvl = vector<vl>;

const ll INFL = 2e18;

struct node {
    int x, a;

    node() = default;
    node(int x) : x(x) {
        a = 0;
    }
};

struct segtree {
    vector<node> t;

    segtree(vector<int>& a) {
        t.resize(4 * a.size());
        build(1, 0, a.size(), a);
    }
    void push(int v) {
        t[2 * v].a += t[v].a;
        t[2 * v + 1].a += t[v].a;
        t[v].a = 0;
    }
    void build(int v, int vl, int vr, vector<int>& a) {
        if (vl + 1 == vr) {
            t[v] = node(a[vl]);
            return;
        }

        build(2 * v, vl, (vl + vr) / 2, a);
        build(2 * v + 1, (vl + vr) / 2, vr, a);
    }
    void upd(int v, int vl, int vr, int l, int r, int d) {
        if (r <= vl || vr <= l) {
            return;
        }
        if (l <= vl && vr <= r) {
            t[v].a += d;
            return;
        }
        push(v);

        upd(2 * v, vl, (vl + vr) / 2, l, r, d);
        upd(2 * v + 1, (vl + vr) / 2, vr, l, r, d);
    }
    int get(int v, int vl, int vr, int i) {
        if (i < vl || vr <= i) {
            return 0;
        }
        if (vl + 1 == vr) {
            return t[v].x + t[v].a;
        }
        push(v);

        return get(2 * v, vl, (vl + vr) / 2, i) +
            get(2 * v + 1, (vl + vr) / 2, vr, i);
    }
};

bool is_par(int rp, int cp, int r, int c) {
    return cp <= c && rp - cp <= r - c;
}

void ans(int r1, int c1, int r2, int c2, int& r3, int& c3, const vector<int>& v) {
    if (r1 == r2 && c1 == c2) {
        r3 = r1;
        c3 = c1;
        return;
    }

    int n = v.size();
    for (int i = n - 1; i >= 0; --i) {
        if (is_par(i, v[i], r1, c1) && is_par(i, v[i], r2, c2)) {
            r3 = i;
            c3 = v[i];
            return;
        }
    }
}

void solve() {
    int n, c1, r1, c2, r2, r3 = 0, c3 = 0;
    cin >> n;
    vi a(n), p(n + 1), v(n);
    for (int i = 0; i < n; ++i) {
        cin >> a[i];
    }
    for (int i = 0; i <= n; ++i) {
        p[i] = i;
    }
    segtree t(p);
    for (int i = 0; i < n; ++i) {
        t.upd(1, 0, n + 1, a[i], n + 1, -1);
        v[a[i] - 1] = t.get(1, 0, n + 1, a[i] - 1);
    }

    int q;
    cin >> q;
    for (; q; --q) {
        cin >> r1 >> c1 >> r2 >> c2;
        ans(r1, c1, r2, c2, r3, c3, v);
        cout << r3 << ' ' << c3 << '\n';
    }
}

int main() {
    ios::sync_with_stdio(0);
    cin.tie(0);
    solve();
    return 0;
}
