#include <iostream>
#include <vector>
#include <set>
#include <algorithm>

const int maxN = 1e5;

struct Item {
	int need, can, upd;
	Item(int need = 0, int can = 0, int upd = 0) : need(need), can(can), upd(upd) {}
} tree[4 * maxN];

struct Query {
	int c, h, id;
	Query(int c = 0, int h = 0, int id = 0) : c(c), h(h), id(id) {}
	bool operator<(const Query& other) const {
		return c < other.c || (c == other.c && h > other.h);
	}
};

bool flag;
int t, n, p[maxN], c[maxN], l[maxN], r[maxN], sz[maxN], it, head[maxN], pos[maxN], inv[maxN], height[maxN];
long long cnt[maxN], ans;
std::vector<int> gr[maxN];
std::set<Query> s;

int dfs1(int u) {
	height[u] = height[p[u]] + 1;
	sz[u] = 1;
	cnt[u] = 0;
	if (gr[u].size() == 0) {
		cnt[u] = l[u];
		ans += 1LL * l[u] * c[u];
	}
	for (int v : gr[u]) {
		sz[u] += dfs1(v);
		cnt[u] += cnt[v];
	}
	return sz[u];
}

void dfs2(int u, int h) {
	head[u] = h;
	pos[u] = it++;
	inv[pos[u]] = u;
	int id = -1;
	for (int v : gr[u])
		if (id == -1 || sz[id] < sz[v])
			id = v;
	if (id != -1)
		dfs2(id, h);
	for (int v : gr[u])
		if (v != id)
			dfs2(v, v);
}

Item unite(const Item& l, const Item& r) {
	return Item(std::max(l.need, r.need), std::min(l.can, r.can));
}

void build(int i, int tl, int tr) {
	if (tr - tl == 1) {
		tree[i] = Item(l[inv[tl]] - cnt[inv[tl]], r[inv[tl]] - cnt[inv[tl]]);
		return;
	}
	int tm = (tl + tr) / 2;
	build(2 * i + 1, tl, tm);
	build(2 * i + 2, tm, tr);
	tree[i] = unite(tree[2 * i + 1], tree[2 * i + 2]);
}

void push(int i, int tl, int tr) {
	if (tree[i].upd == 0)
		return;
	tree[i].need -= tree[i].upd;
	tree[i].can -= tree[i].upd;
	if (tr - tl != 1) {
		tree[2 * i + 1].upd += tree[i].upd;
		tree[2 * i + 2].upd += tree[i].upd;
	}
	tree[i].upd = 0;
}

void update(int i, int tl, int tr, int l, int r, int x) {
	push(i, tl, tr);
	if (l <= tl && tr <= r) {
		tree[i].upd += x;
		push(i, tl, tr);
		return;
	}
	if (r <= tl || tr <= l)
		return;
	int tm = (tl + tr) / 2;
	update(2 * i + 1, tl, tm, l, r, x);
	update(2 * i + 2, tm, tr, l, r, x);
	tree[i] = unite(tree[2 * i + 1], tree[2 * i + 2]);
}

Item get(int i, int tl, int tr, int l, int r) {
	push(i, tl, tr);
	if (l <= tl && tr <= r)
		return tree[i];
	if (r <= tl || tr <= l)
		return Item(-1, 1e9 + 1);
	int tm = (tl + tr) / 2;
	return unite(get(2 * i + 1, tl, tm, l, r), get(2 * i + 2, tm, tr, l, r));
}

int updateHLD(int u) {
	Item res = Item(-1, 1e9 + 1);
	int v = u;
	while (head[v]) {
		res = unite(res, get(0, 0, n, pos[head[v]], pos[v] + 1));
		v = p[head[v]];
	}
	res = unite(res, get(0, 0, n, 0, pos[v] + 1));
	if (res.need <= 0 || res.can <= 0)
		return 0;
	int x = std::min(res.need, res.can);
	v = u;
	while (head[v]) {
		update(0, 0, n, pos[head[v]], pos[v] + 1, x);
		v = p[head[v]];
	}
	update(0, 0, n, 0, pos[v] + 1, x);
	return x;
}

void check(int i, int tl, int tr) {
	push(i, tl, tr);
	if (tr - tl == 1) {
		cnt[inv[tl]] = l[inv[tl]] - tree[i].need;
		return;
	}
	int tm = (tl + tr) / 2;
	check(2 * i + 1, tl, tm);
	check(2 * i + 2, tm, tr);
}

void dfs3(int u) {
	for (int v : gr[u]) {
		cnt[u] -= cnt[v];
		dfs3(v);
	}
}

int main() {
	std::ios_base::sync_with_stdio(false);
	std::cin.tie(0);
	std::cout.tie(0);
	std::cin >> t;
	while (t--) {
		ans = it = 0;
		std::cin >> n;
		for (int i = 0; i < n; i++)
			gr[i].clear();
		for (int i = 1; i < n; i++) {
			std::cin >> p[i];
			gr[--p[i]].push_back(i);
		}
		for (int i = 0; i < n; i++) 
			std::cin >> c[i];
		for (int i = 0; i < n; i++)
			std::cin >> l[i] >> r[i];
		dfs1(0);
		flag = true;
		for (int i = 0; i < n && flag; i++)
			flag = (cnt[i] <= r[i]);
		if (!flag) {
			std::cout << "-1\n";
			continue;
		}
		dfs2(0, 0);
		build(0, 0, n);
		for (int i = 0; i < n; i++)
			s.insert(Query(c[i], height[i], i));
		while (!s.empty()) {
			ans += 1LL * s.begin()->c * updateHLD(s.begin()->id);
			s.erase(s.begin());
		}
		check(0, 0, n);
		for (int i = 0; i < n && flag; i++)
			flag = (cnt[i] >= l[i]);
		if (!flag) {
			std::cout << "-1\n";
			continue;
		}
		dfs3(0);
		std::cout << ans << "\n";
		for (int i = 0; i < n; i++)
			std::cout << cnt[i] << " ";
		std::cout << "\n";
	}
	return 0;
}