#include <iostream>
#include <vector>
#include <algorithm>

const int maxN = 1e5, INF = 1e9 + 1;

struct Item {
	int cost, can, id, upd;
	Item(int cost = 0, int can = 0, int id = 0, int upd = 0) : cost(cost), can(can), id(id), upd(upd) {}
	bool operator<(const Item& other) const {
		return cost < other.cost;
	}
} tree[4 * maxN];

int t, n, p[maxN], c[maxN], l[maxN], r[maxN], sz[maxN], it, pos[maxN], inv[maxN], head[maxN], tout[maxN], ans[maxN];
long long cnt[maxN], res;
std::vector<int> gr[maxN];

int dfs1(int u) {
	sz[u] = 1;
	for (int v : gr[u])
		sz[u] += dfs1(v);
	return sz[u];
}

void dfs2(int u, int h) {
	head[u] = h;
	pos[u] = it++;
	inv[pos[u]] = u;
	int id = -1;
	for (int v : gr[u])
		if (id == -1 || sz[id] < sz[v])
			id = v;
	if (id != -1)
		dfs2(id, h);
	for (int v : gr[u])
		if (v != id)
			dfs2(v, v);
	tout[u] = it;
}

void build(int i, int tl, int tr) {
	if (tr - tl == 1) {
		tree[i] = Item(c[inv[tl]], r[inv[tl]], inv[tl]);
		return;
	}
	int tm = (tl + tr) / 2;
	build(2 * i + 1, tl, tm);
	build(2 * i + 2, tm, tr);
	tree[i] = std::min(tree[2 * i + 1], tree[2 * i + 2]);
}

void push(int i, int tl, int tr) {
	if (tree[i].upd == 0)
		return;
	tree[i].can -= tree[i].upd;
	if (tr - tl != 1) {
		tree[2 * i + 1].upd += tree[i].upd;
		tree[2 * i + 2].upd += tree[i].upd;
	}
	tree[i].upd = 0;
}

Item get1(int i, int tl, int tr, int l, int r) {
	push(i, tl, tr);
	if (l <= tl && tr <= r)
		return tree[i];
	if (r <= tl || tr <= l)
		return Item(INF);
	int tm = (tl + tr) / 2;
	return std::min(get1(2 * i + 1, tl, tm, l, r), get1(2 * i + 2, tm, tr, l, r));
}

Item get2(int i, int tl, int tr, int l, int r) {
	push(i, tl, tr);
	if (l <= tl && tr <= r)
		return tree[i];
	if (r <= tl || tr <= l)
		return Item(0, INF);
	int tm = (tl + tr) / 2;
	Item g1 = get2(2 * i + 1, tl, tm, l, r), g2 = get2(2 * i + 2, tm, tr, l, r);
	if (g1.can < g2.can)
		return g1;
	else
		return g2;
}

Item getHLD(int u) {
	Item res = Item(0, INF);
	while (head[u]) {
		Item temp = get2(0, 0, n, pos[head[u]], pos[u] + 1);
		if (temp.can < res.can)
			res = temp;
		u = p[head[u]];
	}
	Item temp = get2(0, 0, n, 0, pos[u] + 1);
	if (temp.can < res.can)
		res = temp;
	return res;
}

void update1(int i, int tl, int tr, int id) {
	push(i, tl, tr);
	if (tr - tl == 1) {
		tree[i] = Item(INF);
		return;
	}
	int tm = (tl + tr) / 2;
	if (id < tm) {
		update1(2 * i + 1, tl, tm, id);
		push(2 * i + 2, tm, tr);
	}
	else {
		push(2 * i + 1, tl, tm);
		update1(2 * i + 2, tm, tr, id);
	}
	tree[i] = std::min(tree[2 * i + 1], tree[2 * i + 2]);
}

void dfs4(int u) {
	if (c[u] == INF)
		return;
	update1(0, 0, n, pos[u]);
	for (int v : gr[u])
		dfs4(v);
}

void update2(int i, int tl, int tr, int l, int r, int x) {
	push(i, tl, tr);
	if (l <= tl && tr <= r) {
		tree[i].upd += x;
		push(i, tl, tr);
		return;
	}
	if (r <= tl || tr <= l)
		return;
	int tm = (tl + tr) / 2;
	update2(2 * i + 1, tl, tm, l, r, x);
	update2(2 * i + 2, tm, tr, l, r, x);
}

void updateHLD(int u, int x) {
	while (head[u]) {
		update2(0, 0, n, pos[head[u]], pos[u] + 1, x);
		u = p[head[u]];
	}
	update2(0, 0, n, 0, pos[u] + 1, x);
}

bool dfs3(int u) {
	for (int v : gr[u]) {
		if (!dfs3(v))
			return false;
		cnt[u] += cnt[v];
	}
	if (cnt[u] > r[u])
		return false;
	if (cnt[u] >= l[u])
		return true;
	while (cnt[u] < l[u]) {
		Item t1 = get1(0, 0, n, pos[u], tout[u]);
		if (t1.cost == INF)
			return false;
		Item t2 = getHLD(t1.id);
		if (t2.can == 0)
			dfs4(t2.id);
		else {
			int x = std::min(t2.can, l[u] - (int)cnt[u]);
			ans[t1.id] += x;
			res += 1LL * x * t1.cost;
			cnt[u] += x;
			updateHLD(t1.id, x);
		}
	}
	return true;
}

int main() {
	std::ios_base::sync_with_stdio(false);
	std::cin.tie(0);
	std::cout.tie(0);
	std::cin >> t;
	while (t--) {
		std::cin >> n;
		for (int i = 0; i < n; i++) {
			gr[i].clear();
			cnt[i] = ans[i] = 0;
		}
		for (int i = 1; i < n; i++) {
			std::cin >> p[i];
			gr[--p[i]].push_back(i);
		}
		for (int i = 0; i < n; i++) 
			std::cin >> c[i];
		for (int i = 0; i < n; i++)
			std::cin >> l[i] >> r[i];
		dfs1(0);
		dfs2(0, 0);
		build(0, 0, n);
		if (dfs3(0)) {
			std::cout << res << "\n";
			for (int i = 0; i < n; i++)
				std::cout << ans[i] << " ";
			std::cout << "\n";
		}
		else
			std::cout << "-1\n";
	}
	return 0;
}