#include <iostream>
#include <vector>
#include <functional>
#include <cassert>

#define int int64_t

using namespace std;

using graph_t = vector<vector<int>>;

const int ROOT = 0;

const int64_t INF = (1LL << 57);

template <typename T>
vector<T> min_plus(const vector<T> &as, const vector<T> &bs) {
    int n = as.size(), m = bs.size();
    vector<T> res(n + m - 1);
    for (int k = 0; k < n + m - 1; k++) {
        int t = max((int64_t) 0, k - (m - 1));
        res[k] = as[t] + bs[k - t];
        for (int i = 0; i < n; i++) {
            if (k - i < m && k - i >= 0) {
                res[k] = min(res[k], as[i] + bs[k - i]);
            }
        }
    }
    return res;
}

void solve() {
    int n; cin >> n;
    vector<int> parents(n, -1), costs(n);
    graph_t childrens(n);
    for (int i = 1; i < n; i++) {
        cin >> parents[i];
        parents[i]--;
        childrens[parents[i]].push_back(i);
    }
    for (int i = 0; i < n; i++) {
        cin >> costs[i];
    }
    vector<int> ls(n), rs(n);
    for (int i = 0; i < n; i++) {
        cin >> ls[i] >> rs[i];
    }
    
    vector<vector<int64_t>> ans(n);
    function<void(int)> dfs = [&](int v) {
        for (int u : childrens[v]) {
            dfs(u);
        }
        int k = childrens[v].size();
        vector<int> cs(k);
        int64_t cur_ans = 0;
        int cur_amount = 0;
        for (int i = 0; i < k; i++) {
            int u = childrens[v][i];
            cs[i] = ls[u];
            cur_ans += ans[u][cs[i]];
            cur_amount += cs[i];
        }
        vector<int64_t> res(rs[v] + 1, INF);
        if (cur_ans >= INF || cur_amount > rs[v]) {
            ans[v] = res;
            return;
        }
        res[cur_amount] = cur_ans;
        while (cur_amount < rs[v]) {
            int64_t min_delta = costs[v];
            int min_idx = -1;
            for (int i = 0; i < k; i++) {
                int u = childrens[v][i];
                int64_t delta;
                if (ans[u].size() <= cs[i] + 1) {
                    delta = INF;
                } else {
                    delta = ans[u][cs[i] + 1] - ans[u][cs[i]];
                }
                if (delta < min_delta) {
                    min_delta = delta;
                    min_idx = i;
                }
            }
            if (min_idx != -1) {
                cs[min_idx]++;
            }
            cur_amount++;
            cur_ans += min_delta;
            res[cur_amount] = cur_ans;   
        }
        for (int i = 0; i < ls[v]; i++) {
            res[i] = INF;
        }
        ans[v] = res;
    };
    dfs(ROOT);
    int64_t anss = INF;
    int idx = -1;
    for (int j = 0; j < (int) ans[ROOT].size(); j++) {
        int64_t i = ans[ROOT][j];
        if (i < anss) {
            anss = i;
            idx = j;

        }
    }
    if (anss >= INF) {
        cout << -1 << endl;
        return;
    }
    cout << anss << endl;
    vector<int> nums(n, 0);
    function<void(int, int)> recover = [&](int v, int c) {
        int k = childrens[v].size();
        vector<int> cs(k);
        auto get_delta = [&](int i) {
            int u = childrens[v][i];
            if (cs[i] + 1 >= ans[u].size()) return INF;
            return ans[u][cs[i] + 1] - ans[u][cs[i]];
        };
        for (int i = 0; i < k; i++) {
            int u = childrens[v][i];
            cs[i] = ls[u];
            c -= cs[i];
        }
        while (c > 0) {
            int64_t min_delta = costs[v];
            int min_idx = -1;
            for (int i = 0; i < k; i++) {
                int u = childrens[v][i];
                int64_t delta = get_delta(i);
                if (delta < min_delta) {
                    min_delta = delta;
                    min_idx = i;
                }
            }
            if (min_idx != -1) {
                cs[min_idx]++;
            } else {
                nums[v]++;
            }
            c--;
        }
        for (int i = 0; i < k; i++) {
            int u = childrens[v][i];
            recover(u, cs[i]);
        }
    };
    recover(ROOT, idx);
    int64_t total_sum = 0;
    vector<int> subtree_sum(n);
    for (int i = 0; i < n; i++) {
        int v = i;
        int k = nums[v];
        subtree_sum[v] += k;
        while (parents[v] != -1) {
            v = parents[v];
            subtree_sum[v] += k;
        }
    }

    for (int i = 0; i < n; i++) {
        total_sum += costs[i] * 1ll * nums[i];
        assert(ls[i] <= subtree_sum[i] && subtree_sum[i] <= rs[i]);
    }
    assert(total_sum == anss);
    for (int i = 0; i < n; i++) {
        cout << nums[i] << ' ';
    }
    cout << endl;
}

signed main() {
    int t; cin >> t;
    while (t--) {
        solve();
    }
}
