#include <bits/stdc++.h>
using namespace std;

istream& operator >> (istream& in, vector<int>& a) {
    for (int i = 0; i < a.size(); ++i)
        in >> a[i];
    return in;
}

const int N = 1e6;
const int LOG_N = 24;

int up[LOG_N][N];
int tin[N], tout[N];
int depth[N];

int get_progress(int n) {
    return (n * (n + 1)) / 2;
}

int get_vertex(int r, int c) {
    return get_progress(r) + c;
}

vector<vector<int>> build_g(vector<int>& a) {
    vector<bool> down(a.size(), true);
    set<pair<int, int>> edges;
    for (int i = 0; i <= a.size(); ++i) {
        int c = 0;
        for (int d = 0; d < a.size(); ++d) {
            int c2 = c;
            if (down[d]) {
                ++c2;
            }
            edges.insert(make_pair(get_vertex(d, c), get_vertex(d + 1, c2)));
            c = c2;
        }
        if (i < a.size()) {
            down[a[i] - 1] = false;
        }
    }
    vector<vector<int>> g(get_vertex(a.size(), a.size()) + 1);
    for (pair<int, int> e :edges) {
        g[e.first].push_back(e.second);
    }
    return g;
}
int dfs_time = 0;
void build_up(int v, vector<vector<int>>& g, int prev) {
    tin[v] = dfs_time++;
    up[0][v] = prev;
    int k = 0;
    while (up[k][v] != -1) {
        up[k + 1][v] = up[k][up[k][v]];
        ++k;
    }
    if (prev == -1)
        depth[v] = 0;
    else
        depth[v] = depth[prev] + 1;
    while (k < LOG_N) {
        up[k][v] = -1;
        ++k;
    }
    for (int u :g[v]) {
        build_up(u, g, v);
    }
    tout[v] = dfs_time++;
}

bool is_subtree(int vertex, int subtree) {
    return tin[subtree] <= tin[vertex] && tout[vertex] <= tout[subtree];
}

int get_lca(int v1, int v2) {
    int k = LOG_N - 1;
    if (depth[v1] > depth[v2])
        swap(v1, v2);
    if (is_subtree(v2, v1))
        return v1;
    while (k > 0) {
        while (k >= 0 && (up[k][v1] == -1 || is_subtree(v2, up[k][v1])))
            --k;
        //cout << k << ' ' << v1 << endl;
        if (k == -1)
            break;
        v1 = up[k][v1];
    }
    return up[0][v1];
}

pair<int, int> get_cords(int vertex) {
    int left = 0, right = 10001;
    while (right - left > 1) {
        int mid = (right + left) / 2;
        if (get_progress(mid) > vertex)
            right = mid;
        else
            left = mid;
    }
    return make_pair(left, vertex - get_progress(left));
}

void stress(vector<vector<int>> g) {
    for (int i = 0; i < g.size(); ++i) {
        for(int j = 0; j < g.size(); ++j) {
            int lca = get_lca(i, j);
            cout << i << " " << j << ": " << lca << endl;
        }
    }
}

int main()
{
    ios_base::sync_with_stdio(false); cin.tie(0); cout.tie(0);
    //freopen("Cin.txt", "r", stdin);
    //freopen("Cout.txt", "w", stdout);
    int n;
    cin >> n;
    vector<int> a(n);
    cin >> a;
    vector<vector<int>> g = build_g(a);
/*
    for (int i = 0; i < g.size(); ++i) {
        cout << i << ":\n\t";
        for (int v :g[i])
            cout << v << ' ';
        cout << endl;
    }
*/
    build_up(0, g, -1);
    int q;
    cin >> q;
    while (q--) {
        int r1, c1, r2, c2;
        cin >> r1 >> c1 >> r2 >> c2;
        int v1 = get_vertex(r1, c1);
        int v2 = get_vertex(r2, c2);
        int lca = get_lca(v1, v2);
        //cout << v1 << ' ' << v2 << ' ' << lca << endl;
        pair<int, int> ans = get_cords(lca);
        cout << ans.first << ' ' << ans.second << '\n';
    }
}
