#pragma GCC optimize("O3")
#pragma GCC target("avx,avx2")

#include <iostream>
#include <vector>
#include <map>
#include <unordered_map>
#include <set>
#include <unordered_set>
#include <algorithm>
#include <iomanip>
#include <numeric>
#include <cassert>
#include <climits>
#include <random>

using namespace std;

#ifndef DESPAIR_101
#define cerr if (false) cerr
#endif

typedef long long ll;
typedef unsigned long long ull;
typedef long double ld;

template<class A>
void print(A x) {
	cerr << x;
}
template<class A, class B>
void print(pair<A, B> x) {
	cerr << "(" << x.first << ' ' << x.second << ")";
}
template<class A>
void print(vector<A> x) {
	for (A el : x) {
		print(el);
		cerr << ' ';
	}
	cerr << '\n';
}
template<class A>
void print(set<A> x) {
	for (A el : x) {
		print(el);
		cerr << ' ';
	}
	cerr << '\n';
}
template<class A>
void print(unordered_set<A> x) {
	for (A el : x) {
		print(el);
		cerr << ' ';
	}
	cerr << '\n';
}
template<class A, class B>
void print(map<A, B> x) {
	for (A el : x) {
		print(el);
		cerr << ' ';
	}
	cerr << '\n';
}
template<class A, class B>
void print(unordered_map<A, B> x) {
	for (A el : x) {
		print(el);
		cerr << ' ';
	}
	cerr << '\n';
}

//#define int ll
#define sz(x) int(x.size())
#define dbg(x) cerr << #x << ":\n "; print(x); cerr << "\n";

mt19937 rnd(101);

struct Node {
	int x;
	int y;
	int cnt;
	Node* l;
	Node* r;

	Node(int v) {
		x = v;
		y = rnd();
		cnt = 1;
		l = r = nullptr;
	}
};

int getCnt(Node* t) {
	if (!t) return 0;
	return t->cnt;
}

void relax(Node* t) {
	if (!t) return;
	t->cnt = 1 + getCnt(t->l) + getCnt(t->r);
}

pair<Node*, Node*> split(Node* t, int v) {
	// <= & >
	if (!t) return { nullptr, nullptr };
	if (t->x > v) {
		auto [t1, t2] = split(t->l, v);
		t->l = t2;
		relax(t);
		return { t1, t };
	} else {
		auto [t1, t2] = split(t->r, v);
		t->r = t1;
		relax(t);
		return { t, t2 };
	}
}

Node* merge(Node* a, Node* b) {
	if (!a) return b;
	if (!b) return a;
	if (a->y > b->y) {
		a->r = merge(a->r, b);
		relax(a);
		return a;
	} else {
		b->l = merge(a, b->l);
		relax(b);
		return b;
	}
}

Node* insert(Node* t, int v) {
	auto [t1, t2] = split(t, v);
	return merge(t1, merge(new Node(v), t2));
}

void solve() {
	int n;
	cin >> n;

	vector<pair<int, int>> a(n);
	for (int i = 0; i < n; ++i) {
		cin >> a[i].first;
		a[i].second = i;
	}

	Node* t1 = nullptr;

	vector<vector<int>> d(n, vector<int>(n));
	for (int i = 0; i < n; ++i) {
		auto [a1, b1] = split(t1, a[i].first);
		int cnt1 = getCnt(b1);
		t1 = merge(a1, merge(new Node(a[i].first), b1));
		Node* t2 = nullptr;
		for (int j = n - 1; j > i; --j) {
			auto [a2, b2] = split(t2, a[j].first);
			int cnt2 = getCnt(a2);
			t2 = merge(a2, merge(new Node(a[j].first), b2));
			d[i][j] = j - i + max(cnt1, cnt2);
		}
	}

	sort(a.begin(), a.end());
	for (int i = 0; i < n; ++i) {
		for (int j = i + 1; j < n; ++j) {
			int x = min(a[i].second, a[j].second);
			int y = max(a[i].second, a[j].second);
			d[x][y] = max(d[x][y], j - i);
		}
	}

	ll ans = 0;
	for (int i = 0; i < n; ++i) {
		for (int j = i + 1; j < n; ++j) {
			ans += d[i][j];
		}
	}

	cout << ans;
}

signed main() {
	cin.tie(nullptr);
	ios::sync_with_stdio(false);

	int te = 1;
	//cin >> te;

	for (int t = 1; t <= te; ++t) {
		cerr << "_____Test #" << t << "_____\n";
		solve();
	}
}