#include <iostream>
#include <vector>
#include <algorithm>
#include <utility>
#include <numeric>
#include <map>
#include <set>

using namespace std;

#define all(v) v.begin(), v.end()
#define rall(v) v.rbegin(), v.rend()

using uint = unsigned int;
using int64 = long long;
using uint64 = unsigned long long;

const int64 INF = (int64)1e18 + 2;
const int INF32 = (int)1e9 + 2;


class SegmentTree {
private:
	vector <int64> data;
	vector <int64> flag;
	int size;

	void push(int node, int sz) {
		data[node] -= flag[node] * sz;

		if (node < size) {
			flag[node * 2] += flag[node];
			flag[node * 2 + 1] += flag[node];
		}

		flag[node] = 0;
	}

	int64 set(int ql, int qr, int nl, int nr, int node, int64 fl) {
		push(node, nr - nl);
		if (ql >= nr || nl >= qr) {
			return data[node];
		}
		if (ql <= nl && nr <= qr) {
			flag[node] += fl;
			push(node, nr - nl);
			return data[node];
		}

		int nm = (nl + nr) / 2;
		data[node] = min(
			set(ql, qr, nl, nm, node * 2, fl),
			set(ql, qr, nm, nr, node * 2 + 1, fl)
		);
	}

	int64 get(int ql, int qr, int nl, int nr, int node) {
		if (ql >= nr || nl >= qr) {
			return INF32;
		}

		push(node, nr - nl);
		if (ql <= nl && nr <= qr) {
			return data[node];
		}

		int nm = (nl + nr) / 2;
		return min(
			get(ql, qr, nl, nm, node * 2),
			get(ql, qr, nm, nr, node * 2 + 1)
		);
	}

public:
	void reset(const vector <int64>& v) {
		size = 1;
		while (size < v.size()) size *= 2;

		data.assign(size * 2, INF32);
		flag.assign(size * 2, 0);
		for (int i = 0; i < v.size(); ++i) {
			data[i + size] = v[i];
		}

		for (int i = size - 1; i > 0; --i) {
			data[i] = min(data[i * 2], data[i * 2 + 1]);
		}
	}

	void set(int l, int r, int64 value) {
		set(l, r, 0, size, 1, value);
	}

	int64 get(int l, int r) {
		return get(l, r, 0, size, 1);
	}
};

vector <vector <int>> g;
vector <int> cost;
vector <pair <int64, int64>> sg;
vector <int> sz;

int64 checkDfs(int v) {
	int64 mn = 0;
	int size = 1;
	for (auto& u : g[v]) {
		int64 uans = checkDfs(u);
		if (uans == -1) {
			return -1;
		}
		mn += uans;
		size += sz[u];
	}
	sz[v] = size;

	if (mn > sg[v].second) return -1;
	return max(mn, sg[v].first);
}


void heavyDfs(int v) {
	int mx = -1;
	for (auto u : g[v]) {
		if (mx == -1 || sz[u] > sz[mx]) {
			mx = u;
		}
	}
	if (mx != -1) {
		swap(g[v][0], g[v][mx]);
	}
}



int64 ans;
vector <int64> cnt;
vector <int64> can;
vector <int64> rcan;
vector <int> pr;

vector <int> order, rorder, beg;
void paintDfs(int v, bool he) {
	rorder[v] = order.size();
	order.push_back(v);
	beg[rorder[v]] = rorder[v];
	if (he) beg[rorder[v]] = beg[rorder[pr[v]]];

	for (int i = 0; i < g[v].size(); ++i) {
		paintDfs(g[v][i], i == 0);
	}
}

SegmentTree tree;
vector <int> rpr;
int64 getmn(int v) {
	int64 mn = INF32;
	v = rorder[v];
	while (v != 0) {
		if (beg[v] == v) {
			mn = min(mn, rcan[v]);
			v = rpr[v];
		}
		else {
			mn = min(mn, tree.get(beg[v] + 1, v + 1));
			v = beg[v];
		}
	}
	return min(mn, rcan[0]);
}
int64 get(int v) {
	v = rorder[v];
	if (beg[v] == v) {
		return rcan[v];
	}
	return tree.get(v, v + 1);
}
void add(int v, int64 value) {
	v = rorder[v];
	while (v != 0) {
		if (beg[v] == v) {
			rcan[v] -= value;
			v = rpr[v];
		}
		else {
			tree.set(beg[v] + 1, v + 1, value);
			v = beg[v];
		}
	}
	rcan[0] -= value;
}

set <pair <int64, int64>> dfs(int v) {
	int mx = -1;
	for (auto u : g[v]) {
		if (mx == -1 || sz[u] > sz[mx]) {
			mx = u;
		}
	}

	if (mx == -1) {
		set <pair <int64, int64>> st;

		int64 c = get(v);

		int64 mn = getmn(v);
		ans += cost[v] * (c - (sg[v].second - sg[v].first));
		cnt[v] += c - (sg[v].second - sg[v].first);

		add(v, c - (sg[v].second - sg[v].first));
		if (c != sg[v].second - sg[v].first) st.insert({ cost[v], v });
		return st;
	}


	auto st = dfs(mx);
	for (auto u : g[v]) {
		if (u != mx) {
			auto st1 = dfs(u);
			for (auto& x : st1) {
				st.insert(x);
			}
		}
	}
	st.insert({ cost[v], v });

	while (get(v) > sg[v].second - sg[v].first) {
		int64 mn = getmn(st.begin()->second);
		if (mn <= get(v) - (sg[v].second - sg[v].first)) {
			add(st.begin()->second, mn);

			ans += mn * st.begin()->first;
			cnt[st.begin()->second] += mn;

			st.erase(st.begin());
		}
		else {
			mn = get(v) - (sg[v].second - sg[v].first);

			add(st.begin()->second, mn);

			ans += mn * st.begin()->first;
			cnt[st.begin()->second] += mn;
		}
	}

	return st;
}

void solve() {
	int n; cin >> n;
	g.assign(n, {});
	pr.assign(n, -1);
	for (int i = 1; i < n; ++i) {
		int p; cin >> p;
		g[p - 1].push_back(i);
		pr[i] = p - 1;
	}

	cost.assign(n, 0);
	for (auto& x : cost) cin >> x;

	sg.assign(n, { 0, 0 });
	for (auto& [f, s] : sg) cin >> f >> s;

	sz.assign(n, 0);
	if (checkDfs(0) == -1) {
		cout << -1;
		return;
	}

	heavyDfs(0);

	rorder.assign(n, 0);
	beg.assign(n, 0);
	paintDfs(0, false);


	ans = 0;
	cnt.assign(n, 0);
	can.assign(n, 0);
	for (int i = 0; i < n; ++i) {
		can[i] = sg[i].second;
	}

	rcan.assign(n, 0);
	for (int i = 0; i < n; ++i) {
		rcan[rorder[i]] = can[i];
	}
	tree.reset(rcan);

	rpr.assign(n, 0);
	for (int i = 0; i < n; ++i) {
		if (i == 0) rpr[rorder[i]] = -1;
		else rpr[rorder[i]] = rorder[pr[i]];
	}

	dfs(0);

	cout << ans << '\n';
	for (auto x : cnt) cout << x << ' ';
}

int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);
	cout.tie(nullptr);

	int q; cin >> q;
	while (q--) {
		solve();
		cout << "\n\n";
	}
}