#ifndef LOCAL
#pragma GCC optimize("Ofast")
#pragma GCC optimize("O3")
#endif

#include <iostream>
#include <vector>
#include <algorithm>
#include <queue>
#include <numeric>
#include <cmath>

#define len(a) (int)a.size()

typedef long long ll;

const int INF = (int)1e9;

using namespace std;

void run();

int main()
{
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);
#endif

    run();

}

vector< vector<int> > graph;

int get_id(short r, short c)
{
    return (int)r * (r + 1) / 2 + c;
}

vector<short> heights;
vector< vector<int> > up;
int up_size;

void dfs(int v, short height = 0, int p = -1)
{
    heights[v] = height;
    up[v][0] = p;
    for (int i = 1; i < up_size; ++i)
    {
        up[v][i] = up[up[v][i - 1]][i - 1];
    }
    for (int to : graph[v])
    {
        dfs(to, height + 1, v);
    }
}

int lca(int u, int v)
{
    if (heights[u] > heights[v])
    {
        swap(u, v);
    }
    for (int i = up_size - 1; i > -1; --i)
    {
        if (heights[up[v][i]] >= heights[u])
        {
            v = up[v][i];
        }
    }
    if (v == u)
    {
        return v;
    }
    for (int i = up_size - 1; i > -1; --i)
    {
        if (up[v][i] != up[u][i])
        {
            v = up[v][i];
            u = up[u][i];
        }
    }
    return up[u][0];
}

void run()
{
    int n;
    cin >> n;
    vector<int> arr(n);
    for (int i = 0; i < n; ++i)
    {
        cin >> arr[i];
    }
    int count = get_id(n, n) + 1;

    vector<char> prog(n, 1);
    vector<char> used(count);

    auto process_prog = [&]()
    {
        short x, y;
        x = y = 0;
        int v = 0;
        for (int i = 0; i < n; ++i)
        {
            ++x;
            y += prog[i];
            int id = get_id(x, y);
            if (!used[id])
            {
                graph[v].push_back(id);
                used[id] = 1;
            }
            v = id;
        }
    };

    graph.resize(count);
    process_prog();
    for (int i = 0; i < n; ++i)
    {
        prog[arr[i] - 1] = 0;
        process_prog();
    }

    used.resize(0);
    used.shrink_to_fit();
    prog.resize(0);
    prog.shrink_to_fit();
    arr.resize(0);
    arr.shrink_to_fit();

    up_size = ceil(log2(n));
    up.resize(count, vector<int>(up_size));
    heights.resize(count);
    dfs(0, 0, 0);

    int q;
    cin >> q;
    for (; q > 0; --q)
    {
        short r1, c1, r2, c2;
        cin >> r1 >> c1 >> r2 >> c2;

        int id = lca(get_id(r1, c1), get_id(r2, c2));
        int left, right;
        left = 0;
        right = n + 1;
        while (right - left > 1)
        {
            int mid = (left + right) / 2;
            int azaz = mid * (mid + 1) / 2;
            if (azaz <= id)
            {
                left = mid;
            }
            else
            {
                right = mid;
            }
        }

        int azaz = left * (left + 1) / 2;
        cout << left << ' ' << id - azaz << '\n';
    }
}
