//#define _GLIBCXX_DEBUG
//#pragma GCC optimize("Ofast")
//#pragma GCC target("avx2,avx512f,avx512dq,bmi2")
#include <iostream>
#include <random>
#include <vector>
#include <climits>
#include <map>
#include <algorithm>
std::mt19937 mt;

std::array<std::vector<int>, 13> parent;
std::vector<char> insn;
std::vector<int> triag;
std::vector<std::vector<int>> children;
int N = 0;
void dfs(int r, int c, int prev = 0)
{
	// pairing function: triangular numbers plus c
	int i = triag[r] + c;
	parent[0][i] = prev;
	if(r == N) return;
	int nr = r+1, nc = c;
	//printf("r=%d, c=%d\n", r, c);
	if(insn[r] == 1) nc++;
	int ni = triag[nr]+nc;
	if(std::find(children[i].begin(), children[i].end(), ni) == children[i].end())
		children[i].push_back(ni);
	dfs(nr, nc, i);
}
std::vector<int> t_in, t_out;
int cur_t = 0;
void dfs2(int i)
{
	t_in[i] = cur_t++;
	for(int to : children[i])
		dfs2(to);
	t_out[i] = cur_t++;
}

bool is_parent(int i1, int i2)
{
	return t_in[i1] <= t_in[i2] && t_out[i2] <= t_out[i1];
}
struct stree
{
	stree* l, *r;
	int sum;
	void mkl() { if(!l) l = new stree {}; }
	void mkr() { if(!r) r = new stree {}; }
	stree* inc(int at, int L, int R) const
	{
		int M = (L + R) / 2;
		stree* copy = new stree(*this);
		if(M == L)
		{
			copy->sum += 1;
			return copy;
		}
		if(at < M)
		{
			copy->mkl();
			copy->sum += 1;
			copy->l = copy->l->inc(at, L, M);
			return copy;
		}
		else
		{
			copy->mkr();
			copy->sum += 1;
			copy->r = copy->r->inc(at, M, R);
			return copy;
		}
	}
	int rsum(int L, int R, int ql, int qr) const
	{
		if(ql <= L && R <= qr) return sum;
		if(L >= qr || R <= ql) return 0;
		int ans = 0;
		if(l) ans += l->rsum(L, (L+R)/2, ql, qr);
		if(r) ans += r->rsum((L+R)/2, R, ql, qr);
		return ans;
	}
};
const int bound = 131072;
std::vector<stree*> history;
int find_iter(int r, int c)
{
	int L = 0, R = history.size();
	while(L != R)
	{
		int M = (L+R) / 2;
		//printf("total %d\n", history[M]->sum);
		int val = history[M]->rsum(0, bound, 0, r);
		// val = number of left turns
		//printf("M=%d: lt=%d\n", M,val);
		if(r - c >= val) // not enough left turns -> not enough iterations
			L = M + 1;
		else R = M;
	}
	return L - 1;
}
int main()
{
	// binary lifting
	std::cin >> N;
	//triag.resize(N+1);
	//int gsz = (N+2) * (N+1) / 2;
	//parent[0].resize(gsz);
	//for(int i = 1; i < N+1; i++)
	//	triag[i] = i + triag[i-1];

	std::vector<int> a(N);
	//insn.resize(N, 1);
	//t_in.resize(gsz);
	//t_out.resize(gsz);
	//children.resize(gsz);
	for(int& x : a) scanf("%d", &x), x--;
	//for(int i = 0; i < N; i++)
	//{
	//	dfs(0, 0);
	//	insn[a[i]] = 0;
	//}
	//dfs(0, 0);
	//dfs2(0);

	stree* curt = new stree {};
	history.push_back(curt);
	for(int i = 0; i < N; i++)
	{
		curt = curt->inc(a[i], 0, bound);
		history.push_back(curt);
	}
	std::vector<int> near_ans = a; std::reverse(near_ans.begin(), near_ans.end());
	children = {};
	int q; std::cin >> q;
	while(q --> 0)
	{
		int r1,c1,r2,c2;
		scanf("%d%d%d%d", &r1,&c1,&r2,&c2);
		//r1 = mt() % -~N;
		//r2 = mt() % -~N;
		//c1 = mt() % -~r1;
		//c2 = mt() % -~r2;
		if(r1 < r2) std::swap(r1, r2), std::swap(c1, c2);
		//r1 is lower
		int rr, rc;
		for(int _ = 0; _ < 1; _++)
		{
			int it1 = find_iter(r1,c1);
			int it2 = find_iter(r2,c2);
			//printf("%d %d: %d\n", r1, c1, find_iter(r1, c1));
			//printf("%d %d: %d\n", r2, c2, find_iter(r2, c2));
			if(c2 == r2 - history[it1]->rsum(0, bound,0,r2))
			{
				//printf("MATCH\n");
				rr = r2;
				rc = c2;
				continue;
			}
			int nc2 = N - it2;
			int nc1 = N - it1;
			int nr1 = N;
			int nr2 = N;
			//printf("%d %d %d %d\n", nr1, nc1, nr2, nc2);
			// compute height
			int min = INT_MAX;
			for(int i = std::min(nc1, nc2); i < std::max(nc1, nc2); i++)
			{
				min = std::min(min, near_ans[i]);
			}
			//printf("h=%d\n", min);
			rr = min;
			rc = rr - history[it1]->rsum(0, bound, 0, rr);

		}
		//int i1 = triag[r1] + c1;
		//int i2 = triag[r2] + c2;
		//while(!is_parent(i1,i2))
		//	i1 = parent[0][i1];
		//int r = std::lower_bound(triag.begin(), triag.end(), i1+1) - triag.begin() - 1;
		//int c = i1 - triag[r];
		//if(rr != r || rc != c)
		//{
		//	//printf("BAD\n");
		//}
		printf("%d %d\n", rr, rc);
		//printf("%d %d\n", r, c);
	}
}
// persistent segment tree, lower_bound
// find an iteration corresponding to all r,c pairs
//  ^: persistent segment tree? history = prefixes of changes of insn.
//     binary search for first version which has low enough prefix sum until row r
// use that iteration number to descend to the bottom directly (r=N, c=-i)
// rmq -> find the level of lca
// level ancestor by partially simulating the iteration
