#include <bits/stdc++.h>

#define ssize(a) (int)(a).size()
#define all(a) (a).begin(), (a).end()

#define int long long

using namespace std;

using ll = long long;

const int inf = 2e9;


struct MinSegTree {
    vector<pair<int, int>> tree;
    int n;

    MinSegTree() {}

    MinSegTree(vector<int> &a) {
        n = ssize(a);
        tree.resize(2 * n);
        for (int i = 0; i < n; ++i) {
            tree[i + n] = {a[i], i};
        }
        for (int i = n - 1; i > 0; --i) {
            tree[i] = min(tree[2 * i], tree[2 * i + 1]);
        }
    }

    pair<int, int> get(int l, int r) {
        pair<int, int> res = {inf, -1};
        l += n;
        r += n;
        while (l < r) {
            if (l & 1) {
                res = min(res, tree[l++]);
            }
            if (r & 1) {
                res = min(res, tree[--r]);
            }
            l /= 2;
            r /= 2;
        }
        return res;
    }

    void update(int pos, int val) {
        int v = pos + n;
        tree[v].first = val;
        v /= 2;
        while (v > 0) {
            tree[v] = min(tree[2 * v], tree[2 * v + 1]);
            v /= 2;
        }
    }
};

struct HldSegTree {
    struct Node {
        int min;
        int sub;
    };
    vector<Node> tree;
    int n;

    void build(vector<int> &a, int v, int vl, int vr) {
        tree[v].sub = 0;
        if (vr - vl == 1) {
            tree[v].min = a[vl];
            return;
        }
        int m = (vl + vr) / 2;
        build(a, 2 * v, vl, m);
        build(a, 2 * v + 1, m, vr);
        tree[v].min = min(tree[2 * v].min, tree[2 * v + 1].min);
    }

    HldSegTree() {}

    HldSegTree(vector<int> &a) {
        n = ssize(a);
        tree.resize(4 * n);
        build(a, 1, 0, n);
    }

    void push(int v, int vl, int vr) {
        tree[v].min -= tree[v].sub;
        if (vr - vl > 1) {
            tree[2 * v].sub += tree[v].sub;
            tree[2 * v + 1].sub += tree[v].sub;
        }
        tree[v].sub = 0;
    }

    int get(int l, int r, int v, int vl, int vr) {
        push(v, vl, vr);
        if (vl >= r || vr <= l) {
            return inf;
        }
        if (vl >= l && vr <= r) {
            return tree[v].min;
        }
        int m = (vl + vr) / 2;
        return min(get(l, r, 2 * v, vl, m), get(l, r, 2 * v + 1, m, vr));
    }

    int get(int l, int r) {
        return get(l, r, 1, 0, n);
    }

    void update(int l, int r, int dlt, int v, int vl, int vr) {
        push(v, vl, vr);
        if (vl >= r || vr <= l) {
            return;
        }
        if (vl >= l && vr <= r) {
            tree[v].sub += dlt;
            push(v, vl, vr);
            return;
        }
        int m = (vl + vr) / 2;
        update(l, r, dlt, 2 * v, vl, m);
        update(l, r, dlt, 2 * v + 1, m, vr);
    }

    void update(int l, int r, int dlt) {
        update(l, r, dlt, 1, 0, n);
    }
};

int n;
vector<vector<int>> g;
vector<int> p;
vector<int> c;
vector<int> al;
vector<int> ar;
vector<int> tin;
vector<int> tout;
vector<int> euler;
MinSegTree min_st;
vector<int> b;
vector<int> subtree;
vector<int> head;
bool conflict;
int timer;
ll ans;
HldSegTree hld_st;

void dfs1(int v) {
    subtree[v] = 1;

    for (int u: g[v]) {
        dfs1(u);
        subtree[v] += subtree[u];
    }
}

void hld(int v) {
    tin[v] = timer++;
    euler.push_back(v);
    for (int u: g[v]) {
        if (u == g[v][0]) {
            head[u] = head[v];
        } else {
            head[u] = u;
        }
        hld(u);
    }
    tout[v] = timer;
}

void path_update(int v, int dlt) {
    while (v != -1) {
        hld_st.update(tin[head[v]], tin[v] + 1, dlt);
        v = p[head[v]];
    }
}

int path_get(int v) {
    int mn = inf;
    while (v != -1) {
        mn = min(mn, hld_st.get(tin[head[v]], tin[v] + 1));
        v = p[head[v]];
    }
    return mn;
}

void dfs2(int v) {
    for (int u: g[v]) {
        dfs2(u);
    }

    if (conflict) {
        return;
    }

    int v_rest = hld_st.get(tin[v], tin[v] + 1);

    while (ar[v] - v_rest < al[v]) {
        pair<int, int> min_cost = min_st.get(tin[v], tout[v]);
        if (min_cost.first == inf) {
            conflict = true;
            return;
        }
        int u = euler[min_cost.second];

        int min_rest = path_get(u);

        if (min_rest == 0) {
            min_st.update(tin[u], inf);
            continue;
        }

        int need = al[v] - (ar[v] - v_rest);
        int buy = min(need, min_rest);

        ans += (ll) buy * min_cost.first;
        b[u] += buy;

        path_update(u, buy);
        v_rest -= buy;
    }
}


void solve() {
    cin >> n;
    g.assign(n, vector<int>());
    c.assign(n, 0);
    al.assign(n, 0);
    ar.assign(n, 0);
    p.assign(n, 0);
    b.assign(n, 0);
    tin.assign(n, 0);
    tout.assign(n, 0);
    subtree.assign(n, 0);
    head.assign(n, 0);
    euler.clear();
    conflict = false;

    p[0] = -1;
    for (int i = 1; i < n; ++i) {
        cin >> p[i];
        --p[i];
        g[p[i]].push_back(i);
    }
    for (int i = 0; i < n; ++i) {
        cin >> c[i];
    }
    for (int i = 0; i < n; ++i) {
        cin >> al[i] >> ar[i];
    }

    timer = 0;
    dfs1(0);
    for (int v = 0; v < n; ++v) {
        for (int &u: g[v]) {
            if (subtree[u] > subtree[g[v][0]])
                swap(u, g[v][0]);
        }
    }
    hld(0);


    vector<int> reorder_c(n);
    for (int v = 0; v < n; ++v) {
        reorder_c[tin[v]] = c[v];
    }
    min_st = MinSegTree(reorder_c);
    vector<int> reorder_rest(n);
    for (int v = 0; v < n; ++v) {
        reorder_rest[tin[v]] = ar[v];
    }
    hld_st = HldSegTree(reorder_rest);

    ans = 0;
    dfs2(0);

    if (conflict) {
        cout << "-1\n";
    } else {
        cout << ans << "\n";
        for (int v = 0; v < n; ++v) {
            cout << b[v] << " ";
        }
        cout << "\n";
    }
}

signed main() {
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
#endif
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    int t;
    cin >> t;
    while (t--) {
        solve();
    }

    return 0;
}
