#include <bits/stdc++.h>

using namespace std;

#define int long long

class SegmentTreeMin {
	vector<int> a;
	int n;

public:
	SegmentTreeMin() {}
	SegmentTreeMin(const vector<int>& data) {
		n = data.size();
		a.resize(2 * n);
		copy(data.begin(), data.end(), a.begin() + n);
		for(int i = n - 1; i >= 1; i--) {
			a[i] = min(a[i * 2], a[i * 2 + 1]);
		}
	}

	int get(int l, int r) {
		l += n;
		r += n;
		int ans = n;
		while(l < r) {
			ans = min(ans, a[l]);
			ans = min(ans, a[r - 1]);
			l = (l + 1) / 2;
			r /= 2;
		}
		return ans;
	}
};

struct Node {
	int sum;
	int left, right;
};
vector<Node> nodes;

class PersistentSegmentTree {
	int n;
	int root;

public:
	PersistentSegmentTree(int n): n(n) {
		root = -1;
	}
	PersistentSegmentTree(const PersistentSegmentTree& st) {
		n = st.n;
		root = st.root;
	}

	static int set(int v, int vl, int vr, int i, int x) {
		if(vr - vl == 1) {
			nodes.emplace_back(Node{x, -1, -1});
			return nodes.size() - 1;
		} else {
			int left = v == -1 ? -1 : nodes[v].left;
			int right = v == -1 ? -1 : nodes[v].right;
			int vm = (vl + vr) / 2;
			if(i < vm) {
				left = set(left, vl, vm, i, x);
			} else {
				right = set(right, vm, vr, i, x);
			}
			nodes.emplace_back(Node{(left == -1 ? 0 : nodes[left].sum) + (right == -1 ? 0 : nodes[right].sum), left, right});
			return nodes.size() - 1;
		}
	}

	static int get(int v, int vl, int vr, int l, int r) {
		if(v == -1) {
			return 0;
		}
		if(vl == l && vr == r) {
			return nodes[v].sum;
		} else {
			int vm = (vl + vr) / 2;
			if(r <= vm) {
				return get(nodes[v].left, vl, vm, l, r);
			} else if(l >= vm) {
				return get(nodes[v].right, vm, vr, l, r);
			} else {
				return get(nodes[v].left, vl, vm, l, vm) + get(nodes[v].right, vm, vr, vm, r);
			}
		}
	}

	static int prefix_ge(int v, int vl, int vr, int x) {
		if(x == 0) {
			return vl;
		}
		if(vr - vl == 1) {
			assert(x == 1 && nodes[v].sum == 1);
			return vr;
		}
		int vm = (vl + vr) / 2;
		int sum_l = nodes[v].left == -1 ? 0 : nodes[nodes[v].left].sum;
		if(x < sum_l) {
			return prefix_ge(nodes[v].left, vl, vm, x);
		} else {
			return prefix_ge(nodes[v].right, vm, vr, x - sum_l);
		}
	}

	void set(int i, int x) {
		root = set(root, 0, n, i, x);
	}

	int get(int l, int r) {
		return get(root, 0, n, l, r);
	}

	int prefix_ge(int x) {
		return prefix_ge(root, 0, n, x);
	}
};

int n;
vector<int> a;
SegmentTreeMin a_st;
vector<PersistentSegmentTree> st_by_prefix;

pair<int, int> lca(pair<int, int> u, pair<int, int> v) {
	int i_u = u.first == 0 ? 0 : st_by_prefix[u.first - 1].prefix_ge(u.first - u.second);
	int i_v = v.first == 0 ? 0 : st_by_prefix[v.first - 1].prefix_ge(v.first - v.second);
	int r = min({a_st.get(min(i_u, i_v), max(i_u, i_v)), u.first, v.first});
	int c = r - (r == 0 ? 0 : st_by_prefix[r - 1].get(0, i_u));
	return make_pair(r, c);
}

signed main() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);

	cin >> n;

	a.resize(n);
	for(int& x: a) {
		cin >> x;
		x--;
	}

	a_st = SegmentTreeMin(a);

	vector<int> b(n);
	for(int i = 0; i < n; i++) {
		b[a[i]] = i;
	}

	PersistentSegmentTree st(n);
	for(int i = 0; i < n; i++) {
		st.set(b[i], 1);
		st_by_prefix.emplace_back(st);
	}

	int q;
	cin >> q;

	vector<tuple<int, int, int, int>> queries(q);

	for(int i = 0; i < q; i++) {
		int r1, c1, r2, c2;
		cin >> r1 >> c1 >> r2 >> c2;
		queries[i] = {r1, c1, r2, c2};
	}

	for(int i = 0; i < q; i++) {
		auto [r1, c1, r2, c2] = queries[i];
		auto [r3, c3] = lca(make_pair(r1, c1), make_pair(r2, c2));
		cout << r3 << " " << c3 << "\n";
	}

	return 0;
}