#include <iostream>
#include <vector>
#include <algorithm>
#include <string>
#include <set>
#include <stack>
#include <map>
#include <cstdlib>
#include <cassert>

using namespace std;
#pragma GCC optimize("03")

vector<int> gol; // 20mb
vector<int> gor; // 20mb
vector<int> d; // 20mb
vector<vector<int>> ppow; // 13 * 20 = 260mb;

void godwn(const vector<bool>& wayr) {
    int crr = 0;
    int crc = 0;
    for (int i = 0; i < wayr.size(); i++) {
        int cri = (crr * (crr + 1)) / 2 + crc;
        if (wayr[i]) {
            // right
            int tor = crr + 1;
            int toc = crc + 1;
            if (gor[cri] == -1) {
                gor[cri] = ((tor * (tor + 1)) / 2 + toc);
            }
            crr = tor;
            crc = toc;
        } else {
            // left
            int tor = crr + 1;
            int toc = crc;
            if (gol[cri] == -1) {
                gol[cri] = ((tor * (tor + 1)) / 2 + toc);
            }
            crr = tor;
            crc = toc;
        }
    }
}

void dfsbuild(int crr, int crc) {
    int cri = (crr * (crr + 1)) / 2 + crc;
    for (int p = 1; p < 13; p++) {
        if (ppow[cri][p - 1] != -1) {
            ppow[cri][p] = ppow[ppow[cri][p - 1]][p - 1];
        }
    }
    if (gor[cri] != -1) {
        //right
        ppow[((crr + 1) * ((crr + 1) + 1) / 2) + crc + 1][0] = cri;
        d[((crr + 1) * ((crr + 1) + 1) / 2) + crc + 1] = d[cri] + 1;
        dfsbuild(crr + 1, crc + 1);
    }
    if (gol[cri] != -1) {
        //left
        ppow[ ((crr + 1) * ((crr + 1) + 1) / 2) + crc][0] = cri;
        d[ ((crr + 1) * ((crr + 1) + 1) / 2) + crc] = d[cri] + 1;
        dfsbuild(crr + 1, crc);
    }
}

pair<int, int> getindbyi(int cri) {
    if (cri == 0) return {0, 0};
    int l = 0;
    int r = 3200;
    while (r - l > 1) {
        int m = (l + r) / 2;
        if (m * (m + 1) / 2 <= cri) {
            l = m;
        } else {
            r = m;
        }
    }
    return {l, cri - (l * (l + 1) / 2)};
}

int getlca(int a, int b) {
    if (d[a] > d[b]) swap(a, b);
    for (int p = 12; p >= 0; p--) {
        int tob = ppow[b][p];
        if (tob != -1 && d[tob] >= d[a]) b = tob;
    }
    if (a == b) return a;
    for (int p = 12; p >= 0; p--) {
        int toa = ppow[a][p];
        int tob = ppow[b][p];
        if (toa != tob) {
            a = toa;
            b = tob;
        }
    }
    return ppow[a][0];
}

signed main() {
    int n;
    cin >> n;
    n++;
    if (n <= 3003) {
        d.resize(n * (n + 1) / 2, 0);
        gol.resize(n * (n + 1) / 2, -1);
        gor.resize(n * (n + 1) / 2, -1);
        ppow.resize(n * (n + 1) / 2, vector<int>(13, -1));
        vector<bool> wayr(n - 1, true);
        godwn(wayr);
        for (int i = 0; i < n - 1; i++) {
            int v;
            cin >> v;
            v--;
            wayr[v] = false;
            godwn(wayr);
        }
        dfsbuild(0, 0);
        int q;
        cin >> q;
        while (q--) {
            int r1, c1, r2, c2;
            cin >> r1 >> c1 >> r2 >> c2;
            int i1 = (r1 * (r1 + 1)) / 2 + c1;
            int i2 = (r2 * (r2 + 1)) / 2 + c2;
            int i3 = getlca(i1, i2);
            auto pp = getindbyi(i3);
            cout << pp.first << ' ' << pp.second << '\n';
        }
    } else {
        bool do1 = true;
        int k;
        for (int i = 0; i < n - 1; i++) {
            int v;
            cin >> v;
            if (v != i + 1) {
                do1 = false;
                k = i;
            }
        }
        if (do1) {
            int q;
            cin >> q;
            while (q--) {
                int a1, b1;
                cin >> a1 >> b1;
                int strel1 = a1 - b1;

                int a2, b2;
                cin >> a2 >> b2;
                int strel2 = a2 - b2;

                if (strel1 != strel2) {
                    pair<int, int> root1 = {strel1, 0};
                    pair<int, int> root2 = {strel2, 0};
                    auto res = min(root1, root2);
                    cout << res.first << ' ' << res.second << '\n';
                } else {
                    auto res = min(make_pair(a1, b1), make_pair(a2, b2));
                    cout << res.first << ' ' << res.second << '\n';
                }
            }
        } else {
            int q;
            cin >> q;
            while (q--) {
                int a1, b1;
                cin >> a1 >> b1;
                int strel1 = a1 - b1;

                int a2, b2;
                cin >> a2 >> b2;
                int strel2 = a2 - b2;

                if (strel1 <= k && strel2 <= k) {
                    //////////////////////////////////////////////////////
                    if (strel1 != strel2) {
                        pair<int, int> root1 = {strel1, 0};
                        pair<int, int> root2 = {strel2, 0};
                        auto res = min(root1, root2);
                        cout << res.first << ' ' << res.second << '\n';
                    } else {
                        auto res = min(make_pair(a1, b1), make_pair(a2, b2));
                        cout << res.first << ' ' << res.second << '\n';
                    }
                    //////////////////////////////////////////////////////
                } else if ((strel1 <= k && strel2 > k) || (strel2 <= k && strel2 > 1)) {
                    if ((strel2 <= k && strel2 > 1)) {
                        swap(strel1, strel2);
                        swap(a1, a2);
                        swap(b1, b2);
                    }
                    if (strel1 != k) {
                        //////////////////////////////////////////////////////
                        pair<int, int> root1 = {strel1, 0};
                        cout << root1.first << ' ' << root1.second << '\n';
                        //////////////////////////////////////////////////////
                    } else {
                        //////////////////////////////////////////////////////
                        pair<int, int> root2 = {a2 - (strel2 - strel1), b2};
                        pair<int, int> root1 = {a1, b1};
                        auto res = min(root1, root2);
                        cout << res.first << ' ' << res.second << '\n';
                        //////////////////////////////////////////////////////
                    }
                } else {
                    pair<int, int> root1 = {a1 - (strel1 - k), b1};
                    pair<int, int> root2 = {a2 - (strel2 - k), b2};
                    if (root1 != root2) {
                        auto res = min(root1, root2);
                        cout << res.first << ' ' << res.second << '\n';
                    } else {
                        auto res = min(make_pair(a1, b1), make_pair(a2, b2));
                        cout << res.first << ' ' << res.second << '\n';
                    }
                }
            }
        }
    }
    return 0;
}
