#include <iostream>
#include <algorithm>
#include <set>

struct Offer {
    int32_t cost, num, ind;

    Offer(int32_t cost_, int32_t num_, int32_t ind_) {
        cost = cost_;
        num  =num_;
        ind = ind_;
    }

    bool operator<(const Offer& other) const {
        if(cost == other.cost)
            return ind < other.ind;
        return cost < other.cost;
    }
};

void dfs(int32_t v, std::vector<int32_t>* children, std::pair<int32_t, int32_t>* bounds, bool& ok, int32_t* ans,
         std::set<Offer>** offers, int64_t* offer_sums, int32_t* costs) {
    for(int32_t i = 0; i < children[v].size(); i++) {
        dfs(children[v][i], children, bounds, ok, ans, offers, offer_sums, costs);
        if(!ok)
            return;
    }

    std::pair<int32_t, int32_t> bounds_cur = bounds[v];
    for(int32_t i = 0; i < children[v].size(); i++) {
        bounds_cur.first -= bounds[children[v][i]].first;
        bounds_cur.second -= bounds[children[v][i]].first;
    }
    if(bounds_cur.second < 0) {
        ok = false;
        return;
    }
    if(bounds_cur.first < 0)
        bounds[v].first -= bounds_cur.first;

    int32_t fat_child = -1;
    for(int32_t i = 0; i < children[v].size(); i++)
        if(fat_child == -1 || offers[children[v][i]]->size() > offers[fat_child]->size())
            fat_child = children[v][i];
    if(fat_child == -1) {
        offers[v] = new std::set<Offer>;
        offer_sums[v] = 0;
    } else {
        offers[v] = offers[fat_child];
        offer_sums[v] = offer_sums[fat_child];
    }

    for(int32_t i = 0; i < children[v].size(); i++) {
        if(children[v][i] == fat_child)
            continue;
        offer_sums[v] += offer_sums[children[v][i]];
        for(auto it = offers[children[v][i]]->begin(); it != offers[children[v][i]]->end(); it++) {
            offers[v]->insert(*it);
        }
    }

    while(bounds_cur.first > 0) {
        if(offers[v]->empty() || offers[v]->begin()->cost > costs[v]) {
            ans[v] += bounds_cur.first;
            bounds_cur.first -= ans[v];
            bounds_cur.second -= ans[v];
        } else {
            //use the offer
            int32_t num_cur = std::min(bounds_cur.first, offers[v]->begin()->num);
            bounds_cur.first -= num_cur;
            bounds_cur.second -= num_cur;
            ans[offers[v]->begin()->ind] += num_cur;
            offer_sums[v] -= num_cur;

            Offer old = *offers[v]->begin();
            offers[v]->erase(offers[v]->begin());
            old.num -= num_cur;
            offers[v]->insert(old);

            if(offers[v]->begin()->num == 0)
                offers[v]->erase(offers[v]->begin());
        }
    }
    offers[v]->emplace(costs[v], bounds_cur.second, v);

    while(offer_sums[v] > bounds_cur.second) {
        int64_t to_remove = std::min(offer_sums[v] - bounds_cur.second, (int64_t)offers[v]->rbegin()->num);
        offer_sums[v] -= to_remove;

        Offer old = *(offers[v]->rbegin());
        auto it = offers[v]->end();
        it--;
        offers[v]->erase(it);
        if(to_remove < old.num) {
            old.num -= to_remove;
            offers[v]->insert(old);
        }
    }
}

int main() {
    std::ios::sync_with_stdio(false);
    std::cin.tie(0);

    int32_t num_tests;
    std::cin >> num_tests;

    for(int32_t t = 0; t < num_tests; t++) {
        int32_t n;
        std::cin >> n;

        std::vector<int32_t>* children = new std::vector<int32_t>[n];
        for(int32_t i = 1; i < n; i++) {
            int32_t p;
            std::cin >> p;
            p--;

            children[p].push_back(i);
        }

        int32_t* cost = new int32_t[n];
        for(int32_t i = 0; i < n; i++)
            std::cin >> cost[i];

        std::pair<int32_t, int32_t>* borders = new std::pair<int32_t, int32_t>[n];
        for(int32_t i = 0; i < n; i++)
            std::cin >> borders[i].first >> borders[i].second;

        bool ok = true;
        int32_t* ans = new int32_t[n];
        for(int32_t i = 0; i < n; i++)
            ans[i] = 0;
        std::set<Offer>** offers = new std::set<Offer>*[n];
        int64_t* offer_sums = new int64_t[n];
        for(int32_t i = 0; i < n; i++)
            offer_sums[i] = 0;
        dfs(0, children, borders, ok, ans, offers, offer_sums, cost);

        if(!ok) {
            std::cout << -1 << "\n";
            continue;
        }

        int64_t ans_val = 0;
        for(int32_t i = 0; i < n; i++)
            ans_val += (int64_t)cost[i] * ans[i];
        std::cout << ans_val << "\n";
        for(int32_t i = 0; i < n; i++)
            std::cout << ans[i] << " ";
        std::cout << "\n";
    }
    return 0;
}
