//#pragma GCC optimize("Ofast")

#include <algorithm>
#include <iostream>
#include <numeric>
#include <vector>
#include <set>

#define all(x) x.begin(), x.end()

using namespace std;
using ll = long long;

const int MAXN = 500'000;
const int INF = (int)1e9;

struct Node
{
	int l, r, val, toPush;
};

Node nodes[MAXN * 40];
int lastNode = 0;

int update(int i, int l, int r, int ql, int qr)
{
	if (l >= qr || r <= ql)
		return i;
	int cur = ++lastNode;
	auto& node = nodes[cur];
	nodes[cur] = nodes[i];

	if (ql <= l && r <= qr)
	{
		++node.toPush;
		++node.val;
		return cur;
	}

	int m = (l + r) / 2;
	node.l = update(node.l, l, m, ql, qr);
	node.r = update(node.r, m, r, ql, qr);
	node.val = nodes[node.l].val + nodes[node.r].val + node.toPush;
	return cur;
}

int get1(int i, int l, int r, int pos)
{
	if (l + 1 == r)
		return nodes[i].val;

	int m = (l + r) / 2;
	return get1(nodes[i].l, l, m, pos) + get1(nodes[i].r, m, r, pos) + nodes[i].toPush;
}

int get(int i, int l, int r, int pos)
{
	int ans = 0;
	while (l + 1 != r)
	{
		ans += nodes[i].toPush;
		int m = (l + r) / 2;
		if (pos < m)
		{
			i = nodes[i].l;
			r = m;
		}
		else
		{
			i = nodes[i].r;
			l = m;
		}
	}
	return nodes[i].val + ans;
}

int build(int l, int r)
{
	int cur = ++lastNode;
	if (l + 1 == r)
		return cur;

	int m = (l + r) / 2;
	nodes[cur].l = build(l, m);
	nodes[cur].r = build(m, r);

	return cur;
}

int getVersion(int r, int c, const vector<int>& versions)
{
	int n = versions.size() - 1;
	int l = 0, rt = n + 1;
	while (l + 1 != rt)
	{
		int m = (l + rt) / 2;
		if (get(versions[m], 0, n + 1, r) <= r - c)
			l = m;
		else
			rt = m;
	}
	return l;
}

inline bool check(int v1, int v2, int h, const vector<int>& versions, int n)
{
	return get(versions[v1], 0, n + 1, h) == get(versions[v2], 0, n + 1, h);
}

int main()
{
	ios::sync_with_stdio(false);
	cin.tie(0), cout.tie(0);

	int n = 0;
	cin >> n;

	vector<int> a(n);
	for (int i = 0; i < n; ++i)
		cin >> a[i];

	vector<int> versions = { build(0, n + 1) };
	for (int i = 0; i < n; ++i)
		versions.push_back(update(versions.back(), 0, n + 1, a[i], n + 1));

	int q = 0;
	cin >> q;

	int r1 = 0, c1 = 0, r2 = 0, c2 = 0;
	for (int i = 0; i < q; ++i)
	{
		cin >> r1 >> c1 >> r2 >> c2;

		int v1 = getVersion(r1, c1, versions);
		int v2 = getVersion(r2, c2, versions);

		int l = 0, r = min(r1, r2) + 1;
		while(l + 1 < r)
		{
			int m = (l + r) / 2;
			if (check(v1, v2, m, versions, n))
				l = m;
			else
				r = m;
		}
		cout << l << " " << l - get(versions[v1], 0, n + 1, l) << "\n";
	}

	return 0;
}
