#pragma GCC optimize("Ofast")
#pragma GCC optimize("unroll-loops")

#include <algorithm>
#include <iostream>
#include <numeric>
#include <vector>
#include <set>

#define all(x) x.begin(), x.end()

using namespace std;
using ll = long long;

const int MAXN = 500'000;
const int INF = (int)1e9;

struct Node
{
	Node* l, *r;
	int val, toPush;

	Node() : l(nullptr), r(nullptr), val(0), toPush(0) {}
};

Node* update(Node* i, int l, int r, int ql, int qr)
{
	if (l >= qr || r <= ql)
		return i;
	Node* node = new Node(*i);

	if (ql <= l && r <= qr)
	{
		++node->toPush;
		++node->val;
		return node;
	}

	int m = (l + r) / 2;
	node->l = update(node->l, l, m, ql, qr);
	node->r = update(node->r, m, r, ql, qr);
	node->val = node->l->val + node->r->val + node->toPush;
	return node;
}

int get(Node* i, int l, int r, int pos)
{
	int ans = 0;
	while (l + 1 != r)
	{
		ans += i->toPush;
		int m = (l + r) >> 1;
		if (pos < m)
		{
			i = i->l;
			r = m;
		}
		else
		{
			i = i->r;
			l = m;
		}
	}
	return i->val + ans;
}

Node* build(int l, int r)
{
	Node* cur = new Node();
	if (l + 1 == r)
		return cur;

	int m = (l + r) / 2;
	cur->l = build(l, m);
	cur->r = build(m, r);

	return cur;
}

int getVersion(int r, int c, const vector<Node*>& versions)
{
	int n = versions.size() - 1;
	int l = 0, rt = n + 1;
	while (l + 1 != rt)
	{
		int m = (l + rt) >> 1;
		if (get(versions[m], 0, n + 1, r) <= r - c)
			l = m;
		else
			rt = m;
	}
	return l;
}

struct SegTree
{
	vector<int> a;

	SegTree(const vector<int>& v, int n)
	{
		a.resize(4 * n, INF);
		build(0, 0, n, v);
	}

	void build(int i, int l, int r, const vector<int>& vec)
	{
		if (l + 1 == r)
		{
			a[i] = vec[l];
			return;
		}
		int m = (l + r) / 2;
		build(i * 2 + 1, l, m, vec);
		build(i * 2 + 2, m, r, vec);
		a[i] = min(a[i * 2 + 1], a[i * 2 + 2]);
	}

	int get(int i, int l, int r, int ql, int qr)
	{
		if (ql >= r || l >= qr)
			return INF;
		if (ql <= l && r <= qr)
			return a[i];

		int m = (l + r) / 2;
		return min(get(i * 2 + 1, l, m, ql, qr), get(i * 2 + 2, m, r, ql, qr));
	}
};

int main()
{
	ios::sync_with_stdio(false);
	cin.tie(0), cout.tie(0);

	int n = 0;
	cin >> n;

	vector<int> a(n);
	for (int i = 0; i < n; ++i)
		cin >> a[i];

	SegTree segTree(a, n);

	vector<Node*> versions = { build(0, n + 1) };
	for (int i = 0; i < n; ++i)
		versions.push_back(update(versions.back(), 0, n + 1, a[i], n + 1));

	int q = 0;
	cin >> q;

	int r1 = 0, c1 = 0, r2 = 0, c2 = 0;
	for (int i = 0; i < q; ++i)
	{
		cin >> r1 >> c1 >> r2 >> c2;

		int v1 = getVersion(r1, c1, versions);
		int v2 = getVersion(r2, c2, versions);
		int l = segTree.get(0, 0, n, min(v1, v2), max(v1, v2)) - 1;
		l = min(l, min(r1, r2));
		cout << l << " " << l - get(versions[v1], 0, n + 1, l) << "\n";
	}

	return 0;
}
