#include <bits/stdc++.h>

#define F first
#define S second

using namespace std;
using ll = long long;
using vi = vector<int>;
using pi = pair<int, int>;

template<class T> bool ckmin(T& a, const T& b) { return b < a ? a = b, true : false; }
template<class T> bool ckmax(T& a, const T& b) { return a < b ? a = b, true : false; }

struct Fenwick {
    int n;
    vector<int> fnw;

    Fenwick(int n) : n(n), fnw(n) { }

    void inc(int i) {
        for (; i < n; i |= i + 1) fnw[i]++;
    }

    int sum(int i) {
        int ans = 0;
        for (; i >= 0; i = (i&i+1)-1) ans += fnw[i];
        return ans;
    }
};

struct SegmentTree {
    int n;
    vector<int> tsum;

    SegmentTree(int n) : n(n), tsum(4*n) { build(1, 0, n); }

    void build(int v, int vl, int vr) {
        if (vl + 1 != vr) {
            int vm = (vl + vr) / 2;
            build(2*v, vl, vm);
            build(2*v+1, vm, vr);
        }
        tsum[v] = vr - vl;
    }

    void erase(int v, int vl, int vr, int i) {
        if (vl + 1 == vr) {
            tsum[v] = 0;
        } else {
            int vm = (vl + vr) / 2;
            if (i < vm) erase(2*v, vl, vm, i);
            else erase(2*v+1, vm, vr, i);
            tsum[v] = tsum[2*v] + tsum[2*v+1];
        }
    }

    void erase(int i) {
        erase(1, 0, n, i);
    }

    int kth(int v, int vl, int vr, int i) {
        if (vl + 1 == vr) return vl;
        
        int vm = (vl + vr) / 2;
        return i < tsum[2*v] ? kth(2*v, vl, vm, i) 
                             : kth(2*v+1, vm, vr, i - tsum[2*v]);
    }

    int kth(int i) {
        return kth(1, 0, n, i);
    }
};

const int N = 5e5+7;
const int Q = 5e5+7;
const int V = 2 * N;
const int K = 21;

int jmp[K][V], h[V], child[V], qx[2*Q], qy[2*Q], qj[2*Q], qc;
vi go[V];
SegmentTree sgt(V);

int fup(int v, int h) {
    for (int i = 0; h != 0; i++, h >>= 1) {
        if (h&1) v = jmp[i][v];
    }
    return v;
}

int lca(int a, int b) {
    if (h[a] > h[b]) a = fup(a, h[a] - h[b]);
    if (h[b] > h[a]) b = fup(b, h[b] - h[a]);

    if (a == b) return a;

    for (int i = K; i--; ) {
        if (jmp[i][a] != jmp[i][b]) {
            a = jmp[i][a];
            b = jmp[i][b];
        }
    }

    return jmp[0][a];
}

void solve() {
    int n; cin >> n;
    vector<int> a(n);
    for (int& x : a) cin >> x;

    vector<int> x { 0, n }, y { 0, n };
    int vc = 2;
    vector<int> st { 0, 1 };
    vector<int> par(vc);
    par[0] = -1;
    par[1] = 0;

    Fenwick fnw(n+1);

    int cx = n;
    for (int h : a) {
        cx--;
        while (st.size() > 1 && x[st[st.size() - 2]] >= h-1) st.pop_back();

        fnw.inc(h);

        if (x[st.back()] != h-1) {
            par[st.back()] = vc; st.pop_back();
            par.push_back(st.back());
            x.push_back(h-1);
            y.push_back(h-1 - fnw.sum(h-1));
            st.push_back(vc++);
        }

        par.push_back(st.back());
        y.push_back(cx);
        x.push_back(n);
        st.push_back(vc++);
    }

    for (int i = 1; i < vc; ++i) go[par[i]].push_back(i);

    for (int i = 0; i < vc; ++i) {
        // cout << "#" << i << " => (" << x[i] << ", " << y[i] << ") => " << par[i] << endl;
    }

    queue<int> q;
    q.push(0);

    while (!q.empty()) {
        int v = q.front(); q.pop();
        for (int to : go[v]) {
            h[to] = h[v] + 1;
            q.push(to);
        }
    }

    for (int i = 0; i < vc; ++i) {
        if (x[i] == n) child[y[i]] = i;
    }

    par[0] = 0;
    for (int i = 0; i < vc; ++i) jmp[0][i] = par[i];
    for (int j = 1; j < K; ++j) {
        for (int i = 0; i < vc; ++i) {
            jmp[j][i] = jmp[j-1][jmp[j-1][i]];
        }
    }

    vector<int> ord(vc);
    iota(ord.begin(), ord.end(), 0);
    sort(ord.begin(), ord.end(), [&](int i, int j) { return x[i] > x[j]; });

    cin >> qc;
    for (int i = 0; i < qc; ++i) {
        cin >> qx[2*i] >> qy[2*i] >> qx[2*i+1] >> qy[2*i+1];
    }

    vector<int> qord(2*qc);
    iota(qord.begin(), qord.end(), 0);
    sort(qord.begin(), qord.end(), [&](int i, int j) { return qx[i] > qx[j]; });

    auto it = ord.begin();
    for (int i : qord) {
        while (it != ord.end() && x[*it] >= qx[i]) {
            if (x[*it] != n) {
                sgt.erase(sgt.kth(y[*it]));
                // cout << "Erase " << y[*it] << endl;
            }

            ++it;
        }

        // cout << "Kth " << qx[i] << " : " << qy[i] << " --> " << sgt.kth(qy[i]) << endl;

        qj[i] = child[sgt.kth(qy[i])];
    }

    for (int i = 0; i < qc; ++i) {
        int v = lca(qj[2*i], qj[2*i+1]);
        // cout << "Lca(" << qj[2*i] << ", " << qj[2*i+1] << ") = " << v << endl;

        int ax = x[v], ay = y[v];
        if (qx[2*i] < ax) ax = qx[2*i], ay = qy[2*i];
        if (qx[2*i+1] < ax) ax = qx[2*i+1], ay = qy[2*i+1];
        cout << ax << " " << ay << endl;
    }
}

int main() {
    ios::sync_with_stdio(0); cin.tie(0);

    int t = 1;
    // cin >> t;
    while (t--) solve();
}