#include <iostream>
#include <vector>
#include <algorithm>
#include <cmath>
#include <set>
#include <map>
#include <iomanip>
#include <numeric>
#include <cassert>

using namespace std;
#define int long long
#define ld long double
#define debug(x) cout << #x << " is " << x << '\n';

const int mod = 1e9 + 7, N = 10;

int n, CNT = 0;
vector <vector <int>> g;
vector <set <pair <int, int>>> subTree;
vector <int> lx, rx, c, par, ans, md, tin, tout;

struct Fenwick {
	vector <int> t;
	Fenwick(int n = 0) {
		t.resize(n);
	}
	int get(int r) {
		int res = 0;
		for (; r >= 0; r &= r + 1, --r)
			res += t[r];
		return res;
	}
	void add(int i, int x) {
		for (; i < t.size(); i |= i + 1)
			t[i] += x;
	}
} tree;

void pl(int v, int x) {
	tree.add(tin[v], x);
}

int getCnt(int v) {
	return tree.get(tout[v] - 1) - tree.get(tin[v] - 1);
}

int countDiff(int v) {
	if (v == 1) return rx[v] - getCnt(v);
	return min(rx[v] - getCnt(v), countDiff(par[v]));
}

int timer = 0;
void dfsTime(int v) {
	tin[v] = timer++;
	for (auto& u : g[v])
		dfsTime(u);
	tout[v] = timer;
}

bool dfs(int v) {
	subTree[v].insert({ c[v], v });
	for (auto& u : g[v]) {
		if (!dfs(u)) return false;
	}
	sort(g[v].begin(), g[v].end(), [&](int i, int j) {
		return subTree[i].size() > subTree[j].size();
		});
	for (auto& u : g[v]) {
		if (subTree[u].size() > subTree[v].size())
			subTree[u].swap(subTree[v]);
		for (auto& x : subTree[u])
			subTree[v].insert(x);
	}
	if (getCnt(v) > rx[v]) return false;
	while (getCnt(v) < lx[v]) {
		auto it = subTree[v].begin();
		if (it == subTree[v].end()) break;
		int d = countDiff(it->second);
		CNT++;
		assert(CNT <= 1000000);
		if (d == 0) {
			subTree[v].erase(it);
		}
		else {
			ans[it->second] += min(lx[v] - getCnt(v), d);
			pl(it->second, min(lx[v] - getCnt(v), d));
		}
	}
	if (getCnt(v) < lx[v]) return false;
	return true;
}


signed main() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);
	cout << fixed << setprecision(6);

	int tt; cin >> tt;
	while (tt-- > 0) {
		cin >> n;
		timer = 0;
		CNT = 0;
		tin.assign(n + 1, 0);
		tout.assign(n + 1, 0);
		g.assign(n + 1, {});
		lx.assign(n + 1, 0);
		rx.assign(n + 1, 0);
		c.assign(n + 1, 0);
		par.assign(n + 1, 0);
		ans.assign(n + 1, 0);
		subTree.assign(n + 1, {});
		md.assign(n + 1, 0);
		tree = Fenwick(n + 1);
		for (int i = 2; i <= n; ++i) {
			int p; cin >> p;
			g[p].push_back(i);
			par[i] = p;
		}
		for (int i = 1; i <= n; ++i) cin >> c[i];
		for (int i = 1; i <= n; ++i)
			cin >> lx[i] >> rx[i];
		dfsTime(1);
		if (!dfs(1)) {
			cout << "-1\n";
			continue;
		}
		int sum = 0;
		for (int i = 1; i <= n; ++i)
			sum += c[i] * ans[i];
		cout << sum << '\n';
		for (int i = 1; i <= n; ++i)
			cout << ans[i] << ' ';
		cout << '\n';
	}

}