#include <iostream>
#include <vector>
#include <algorithm>
#include <cmath>
#include <set>
#include <map>
#include <iomanip>
#include <numeric>
#include <cassert>

using namespace std;
#define int long long
#define ld long double
#define debug(x) cout << #x << " is " << x << '\n';

const int mod = 1e9 + 7, N = 10;

int n, CNT = 0;
vector <vector <int>> g;
vector <set <pair <int, int>>> subTree;
vector <int> lx, rx, c, par, ans, md, tin, tout, sz, root;

struct SegTree {
	int size;
	vector <int> tree;
	vector <int> alt;
	SegTree(int n = 0) {
		size = 1;
		while (size < n) size <<= 1;
		tree.resize(2 * size - 1);
		alt.resize(2 * size - 1);
	}
	void push(int v) {
		tree[2 * v + 1] += alt[v];
		tree[2 * v + 2] += alt[v];
		alt[2 * v + 1] += alt[v];
		alt[2 * v + 2] += alt[v];
		alt[v] = 0;
	}
	void upd(int v, int lx, int rx, int l, int r, int x) {
		if (l <= lx and rx <= r) {
			tree[v] += x;
			alt[v] += x;
			return;
		}
		if (rx <= l or r <= lx) return;
		int m = (lx + rx) / 2;
		push(v);
		upd(2 * v + 1, lx, m, l, r, x);
		upd(2 * v + 2, m, rx, l, r, x);
		tree[v] = min(tree[2 * v + 1], tree[2 * v + 2]);
	}
	void upd(int l, int r, int x) {
		upd(0, 0, size, l, r, x);
	}
	int get(int v, int lx, int rx, int l, int r) {
		if (l <= lx and rx <= r) return tree[v];
		if (rx <= l or r <= lx) return mod * mod;
		int m = (lx + rx) / 2;
		push(v);
		return min(
			get(2 * v + 1, lx, m, l, r),
			get(2 * v + 2, m, rx, l, r)
		);
	}
	int get(int l, int r) {
		return get(0, 0, size, l, r);
	}
} ST;

void dfsReorder(int v) {
	sz[v] = 1;
	for (auto& u : g[v]) {
		dfsReorder(u);
		sz[v] += sz[u];
		if (sz[u] > sz[g[v][0]])
			swap(u, g[v][0]);
	}
}

int timer = 0;
void dfsRoot(int v) {
	tin[v] = timer++;
	for (auto& u : g[v]) {
		if (u == g[v][0]) root[u] = root[v];
		else root[u] = u;
		dfsRoot(u);
	}
	tout[v] = timer;
}

void pl(int v, int x) {
	while (true) {
		int L = tin[root[v]], R = tin[v];
		ST.upd(L, R + 1, -x);
		if (root[v] == 1) break;
		v = par[root[v]];
	}
}

int getDiff(int v) {
	int res = mod * mod;
	while (true) {
		int L = tin[root[v]], R = tin[v];
		res = min(res, ST.get(L, R + 1));
		if (root[v] == 1) break;
		v = par[root[v]];
	}
	return res;
}

int getVal(int v) {
	return rx[v] - ST.get(tin[v], tin[v] + 1);
}


bool dfs(int v) {
	subTree[v].insert({ c[v], v });
	for (auto& u : g[v]) {
		if (!dfs(u)) return false;
	}
	sort(g[v].begin(), g[v].end(), [&](int i, int j) {
		return subTree[i].size() > subTree[j].size();
		});
	for (auto& u : g[v]) {
		if (subTree[u].size() > subTree[v].size())
			subTree[u].swap(subTree[v]);
		for (auto& x : subTree[u])
			subTree[v].insert(x);
	}
	if (getVal(v) > rx[v]) return false;
	while (getVal(v) < lx[v]) {
		auto it = subTree[v].begin();
		if (it == subTree[v].end()) break;
		int d = getDiff(it->second);
		if (d == 0) {
			subTree[v].erase(it);
		}
		else {
			ans[it->second] += min(lx[v] - getVal(v), d);
			pl(it->second, min(lx[v] - getVal(v), d));
		}
	}
	if (getVal(v) < lx[v]) return false;
	return true;
}


signed main() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);
	cout << fixed << setprecision(6);

	int tt; cin >> tt;
	while (tt-- > 0) {
		cin >> n;
		timer = 0;
		CNT = 0;
		tin.assign(n + 1, 0);
		tout.assign(n + 1, 0);
		g.assign(n + 1, {});
		lx.assign(n + 1, 0);
		rx.assign(n + 1, 0);
		c.assign(n + 1, 0);
		par.assign(n + 1, 0);
		ans.assign(n + 1, 0);
		subTree.assign(n + 1, {});
		md.assign(n + 1, 0);
		sz.assign(n + 1, 0);
		root.assign(n + 1, 1);
		for (int i = 2; i <= n; ++i) {
			int p; cin >> p;
			g[p].push_back(i);
			par[i] = p;
		}
		for (int i = 1; i <= n; ++i) cin >> c[i];
		for (int i = 1; i <= n; ++i)
			cin >> lx[i] >> rx[i];
		dfsReorder(1);
		dfsRoot(1);
		ST = SegTree(n + 1);
		for (int i = 1; i <= n; ++i) {
			ST.upd(tin[i], tin[i] + 1, rx[i]);
		}
		if (!dfs(1)) {
			cout << "-1\n";
			continue;
		}
		int sum = 0;
		for (int i = 1; i <= n; ++i)
			sum += c[i] * ans[i];
		cout << sum << '\n';
		for (int i = 1; i <= n; ++i)
			cout << ans[i] << ' ';
		cout << '\n';
	}

}