#include <iostream>
#include <vector>
#include <functional>
#include <deque>

#pragma GCC optimize("O3")
#pragma GCC target("avx,avx2,sse,sse2")

using namespace std;
using graph_t = vector<vector<int>>;

const int INF = 1000000007;

struct DSU {
    vector<int> parents, sizes;

    DSU(int n) {
        parents.resize(n);
        sizes.assign(n, 1);
        for (int i = 0; i < n; i++) {
            parents[i] = i;
        }
    }

    int find(int v) {
        if (parents[v] == v) {
            return v;
        }
        int u = find(parents[v]);
        parents[v] = u;
        return u;
    }

    void merge(int v, int u) {
        v = find(v);
        u = find(u);
        if (v == u) {
            return;
        }
        if (sizes[v] >= sizes[u]) swap(v, u);
        parents[v] = u;
        sizes[u] += sizes[v];
    }
};

void fill_transfers(const graph_t &graph, int t, vector<vector<int>> &vcmps) {
    // Excluding t from graph
    int n = graph.size();
    // vector<int> cmps(n, -1);
    // int timer = 0;
    // function<void(int)> fill_cmps = [&](int v) {
    //     if (v == t) return;
    //     cmps[v] = timer;
    //     for (int u : graph[v]) {
    //         if (cmps[u] != timer) {
    //             fill_cmps(u);
    //         }
    //     }
    // };
    // for (int i = 0; i < n; i++) {
    //     if (cmps[i] == -1) {
    //         fill_cmps(i);
    //         timer++;
    //     }
    // }
    // cmps[t] = timer;
    // timer++;
    DSU dsu(n);
    for (int v = 0; v < n; v++) {
        if (v == t) continue;
        for (int u : graph[v]) {
            if (u != t) {
                dsu.merge(v, u);
            }
        }
    }
    for (int i = 0; i < n; i++) {
        vcmps[t][i] = dsu.find(i);
    }
}


void solve() {
    int n, m; cin >> n >> m;
    graph_t graph(n);
    for (int i = 0; i < m; i++) {
        int v, u; cin >> v >> u; v--; u--;
        graph[u].emplace_back(v);
        graph[v].emplace_back(u);
    }
    int max_states = n * n;
    vector<vector<int>> vcmps(n);
    for (int i = 0; i < n; i++) {
        fill_transfers(graph, i, vcmps);
    }
    DSU dsu(max_states);
    for (int v = 0; v < n; v++) {
        vector<int> delegator(n);
        for (int u = 0; u < n; u++) {
            delegator[vcmps[v][u]] = v * n + u;
        }
        for (int u = 0; u < n; u++) {
            dsu.merge(v * n + u, delegator[vcmps[v][u]]);
        }
    }
    int state_cmps_number = max_states;
    vector<vector<int>> compressed_state_graph(state_cmps_number);
    // set<pair<int, int>> edges;
    // Add non-free transfers
    for (int v = 0; v < n; v++) {
        int a = dsu.find(v * n + v);
        for (int u : graph[v]) {
            int b = dsu.find(u * n + v);
            // if (edges.find({a, b}) != edges.end()) continue;
            // edges.insert({a, b});
            compressed_state_graph[a].emplace_back(b);
            compressed_state_graph[b].emplace_back(a);
        }
    }
    // find shortest paths
    vector<int> dists(state_cmps_number, INF);
    dists[dsu.find(0)] = 0;
    deque<int> q = {dsu.find(0)};
    while (!q.empty()) {
        int v = q.front();
        q.pop_front();
        for (int u : compressed_state_graph[v]) {
            if (dists[u] > dists[v] + 1) {
                dists[u] = dists[v] + 1;
                q.push_back(u);
            }
        }
    }
    for (int i = 1; i < n; i++) {
        int d = dists[dsu.find(i * n + i)];
        if (d == INF) {
            cout << -1 << ' ';
        } else {
            cout << d << ' ';
        }
    }
    cout << '\n';
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    int t; cin >> t;
    while (t--) {
        solve();
    }
}
