#include <iostream>
#include <vector>
#include <algorithm>
#include <set>
#include <functional>

using namespace std;
#define int long long

const int S = (1LL << 41);

struct Tree {
  struct Node {
    Node *l, *r;
    int sum, push;
    Node() {
      l = r = nullptr;
      sum = push = 0;
    }
  };

  Node *root;

  Tree() {
    root = new Node();
  }

  void create(Node *x) {
    if (x->l == nullptr) {
      x->l = new Node();
    }
    if (x->r == nullptr) {
      x->r = new Node();
    }
  }

  void propagate(Node *x, int lx, int rx) {
    if (lx + 1 == rx || x->push == 0) return;
    int p = x->push;
    create(x);
    int m = (lx + rx) / 2;
    x->l->push += p;
    x->r->push += p;
    x->l->sum += p * (m - lx);
    x->r->sum += p * (rx - m);
    x->push = 0;
  }

  void add(int l, int r, int val, Node *x, int lx, int rx) {
    if (lx >= r || rx <= l) return;
    propagate(x, lx, rx);
    if (lx >= l && rx <= r) {
      x->sum += (rx - lx) * val;
      x->push = val;
//      cout << lx << ' ' << rx << ' ' << x->sum << '\n';
      return;
    }
    int m = (lx + rx) / 2;
    create(x);
    add(l, r, val, x->l, lx, m);
    add(l, r, val, x->r, m, rx);
    x->sum = x->r->sum + x->l->sum;
  }

  void add(int l, int r, int val) {
//    cout << "add: " << l << ' ' << r << ' ' << val << '\n';
    add(l, r, val, root, 0, S);
  }

  int get(int k, Node *x, int lx, int rx) {
    if (lx >= k || x == nullptr) return 0;
//    cout << lx << ' ' << rx << ' ' << x->sum << ' ' << x->push << '\n';
    propagate(x, lx, rx);
    if (rx <= k) return x->sum;
    int m = (lx + rx) / 2;
    return get(k, x->l, lx, m) + get(k, x->r, m, rx);
  }
};

//struct SegTree {
//  vector<int> tree, push;
//  int s;
//
//  SegTree(int n) {
//    s = 1;
//    while (s < n) s *= 2;
//    tree.resize(2 * s + 1);
//    push.resize(2 * s + 1);
//  }
//
//  void add(int l, int r, int x, int lx, int rx) {
//    if (lx >= r || rx <= l) return;
//    if (lx >= l && rx <= r) {
//
//    }
//  }
//};

void solve() {
  int n, q;
  cin >> n >> q;
  vector<int> a(n);
  for (auto &x : a) cin >> x;

  vector<int> p(n);
  for (int i = 0; i < n; i++) p[i] = i;
  sort(p.begin(), p.end(), [&] (const int& i, const int& j) {
    return a[i] < a[j];
  });
  vector<int> t(n);
  for (int i = 0; i < n; i++) {
    t[p[i]] = i;
  }

  sort(a.begin(), a.end());

  vector<vector<pair<int, int>>> here(n);

  vector<array<int, 3>> queries;
  for (int w = 0; w < q; w++) {
    int type, k;
    cin >> type >> k;
    if (type == 3) {
      int pos;
      cin >> pos;
      --pos;
      here[t[pos]].emplace_back(k, w);
    } else {
      queries.push_back({type, k, w});
    }
  }
  sort(queries.begin(), queries.end(), [&] (const array<int, 3>& lhs, const array<int, 3>& rhs) {
    return lhs[1] < rhs[1];
  });
  vector<int> ans(q);

  Tree st;
  int blocks = 0;

  vector<pair<int, int>> deltas;
  auto add_delta = [&] (int delta, int i) {
    st.add(0, S, 1);
    st.add(delta, S, -1);
    deltas.emplace_back(delta, i);
  };

  Tree st1;
  for (int i = n - 1; i >= 0; ) {
    if (i + 1 < n) {
      add_delta(a[i + 1] - a[i], i);
    }

    int j = i - 1;
    while (j >= 0 && a[j] == a[i]) j--;

    for (int pos = i; pos > j; pos--) {
      for (auto &[k, w] : here[pos]) {
        ans[w] = a[pos] + st.get(k, st.root, 0, S);
      }
    }

    st1.add(j + 1, i + 1, blocks);

    i = j;
    blocks++;
  }

  int ptr = 0;
  sort(deltas.begin(), deltas.end());

//  vector<int> p(n), l(n), r(n);
//  for (int i = 0; i < n; i++) {
//    p[i] = l[i] = r[i] = i;
//  }
//
//  function<int(int)> get = [&] (int v) {
//    return (p[v] == v ? v : p[v] = get(p[v]));
//  };
//
//  auto unite = [&] (int a, int b) {
//    a = get(a);
//    b = get(b);
//    l[a] = min(l[a], l[b]);
//    r[a] = max(r[a], r[b]);
//    p[b] = a;
//  };
//
//  for (int i = 0; i + 1 < n; i++) {
//    if (a[i] == a[i + 1]) {
//      unite(i, i + 1);
//    }
//  }

  int add = st1.root->sum, sum = 0;
//  cout << "add: " << add << '\n';
  if (!deltas.empty()) {
    while (ptr < (int) queries.size() && queries[ptr][1] < deltas[0].first) {
      if (queries[ptr][0] == 1) {
        ans[queries[ptr][2]] = blocks;
      } else {
        ans[queries[ptr][2]] = queries[ptr][1] * add;
      }
      ptr++;
    }
    sum = deltas[0].first * add;
  }

//  cout << "sum: " << sum << '\n';

  int cur_blocks = blocks;
  for (int i = 0; i < (int) deltas.size(); i++) {
    cur_blocks--;
    int j = i + 1;
    while (j < (int) deltas.size() && deltas[i] == deltas[j]) j++;
    for (int k = i; k < j; k++) {
      st1.add(0, deltas[k].second + 1, -1);
    }
    i = j - 1;
    add = st1.root->sum;
//    cout << "add i: " << i << ' ' << add << '\n';
    int nxt = (i + 1 < (int) deltas.size() ? deltas[i + 1].first : S);
    while (ptr < (int) queries.size() && queries[ptr][1] < nxt) {
      if (queries[ptr][0] == 1) {
        ans[queries[ptr][2]] = cur_blocks;
      } else {
        ans[queries[ptr][2]] = sum + add * (queries[ptr][1] - deltas[i].first);
      }
      ptr++;
    }
    sum += add * (nxt - deltas[i].first);
  }

//  int sum = 0, cur_blocks = blocks, prv = 0;
//  for (int i = 0; i < (int) deltas.size(); i++) {
//    cout << i << ' ' << cur_blocks << ' ' << sum << '\n';
//    while (ptr < (int) queries.size() && queries[ptr][1] < deltas[i]) {
//      if (queries[ptr][0] == 1) {
//        ans[queries[ptr][2]] = cur_blocks;
//      } else {
//        ans[queries[ptr][2]] = sum + (cur_blocks * (cur_blocks - 1) / 2) * (queries[ptr][1] - prv);
//      }
//      ptr++;
//    }
//    sum += (cur_blocks * (cur_blocks - 1) / 2) * (deltas[i] - prv);
//    prv = deltas[i];
//    cur_blocks--;
//  }
//
//  while (ptr < (int) queries.size()) {
//    if (queries[ptr][0] == 1) {
//      ans[queries[ptr][2]] = 1;
//    } else {
//      ans[queries[ptr][2]] = sum;
//    }
//    ptr++;
//  }

  for (auto &x : ans) cout << x << ' ';
//
//  vector<int> p(n), end(n);
//  for (int i = 0; i < n; i++) {
//    p[i] = i;
//    end[i] = i;
//  }
//

//
//  set<pair<int, int>> vals;
//  auto unite = [&] (int a, int b) {
//    a = get(a), b = get(b);
//    if (a > b) swap(a, b);
//    p[b] = a;
//    end[a] = end[b];
//  };
//
//  int cur = 0;
//  set<pair<int, int>> vals, deltas;
//  while (!deltas.empty()) {
//    int nxt = cur + ;
//  }
}

signed main() {
  ios_base::sync_with_stdio(0);
  cin.tie(0);
  solve();
  return 0;
}
