#include <bits/stdc++.h>

typedef long long ll;
typedef long double ld;
using namespace std;

struct Query {
    int index, t, k, i;
    Query(int index, int t, int k): index(index), t(t), k(k) {}
    Query(int index, int t, int k, int i): index(index), t(t), k(k), i(i) {}
};

struct Fenwick {
    int n;
    vector <int> tree;

    Fenwick(int n): n(n), tree(n) {}

    void add(int r, int dt) {
        for (; r < n; r |= (r + 1))
            tree[r] += dt;
    }

    int sum(int r) {
        int result = 0;
        for (; r >= 0; r = (r & (r + 1)) - 1)
            result += tree[r];
        return result;
    }

    int sum(int l, int r) {
        return sum(r) - sum(l - 1);
    }
};

void solve() {
    int n, q;
    cin >> n >> q;

    vector <ll> exp(n);
    for (ll &x: exp) cin >> x;

    auto step = [&] (vector <ll> &e) {
        set <int> seen;
        vector <int> add(n);

        vector <ll> to_be_compressed = e;
        sort(to_be_compressed.begin(), to_be_compressed.end());
        to_be_compressed.resize(unique(to_be_compressed.begin(), to_be_compressed.end()) - to_be_compressed.begin());

        int n_compressed = to_be_compressed.size();
        map <ll, int> compressed;
        for (int i = 0; i < n_compressed; i++)
            compressed[to_be_compressed[i]] = i;

        Fenwick tree(n_compressed);
        for (ll x: e) {
            tree.add(compressed[x], seen.find(x) == seen.end());
            seen.insert(x);
        }

        for (int i = 0; i < n; i++) {
            add[i] = tree.sum(compressed[e[i]] + 1, n_compressed - 1);
        }


        for (int i = 0; i < n; i++) {
            e[i] += add[i];
        }

        set <int> ds;
        for (ll x:e) ds.insert(x);

        return make_pair(accumulate(add.begin(), add.end(), 0ll), ds.size());
    };

    int t, k, u;
    vector <Query> queries;
    for (int i = 0; i < q; i++) {
        cin >> t >> k;
        if (t == 3) {
            cin >> u;
            queries.emplace_back(i, t, k, u - 1);
        } else {
            queries.emplace_back(i, t, k);
        }
    }

    sort(queries.begin(), queries.end(), [] (Query lhs, Query rhs) {
        return lhs.k < rhs.k;
    });

    vector <ll> answers(q);

    int kx = 0; ll s = 0, d = 0;

    set <int> seen;
    for (ll x: exp) seen.insert(x);
    d = seen.size();

    for (auto &query: queries) {
        while (kx < query.k) {
            auto [asum, diffs] = step(exp);
            s += asum;
            d = diffs;
            kx++;
        }

        if (query.t == 3) {
            answers[query.index] = exp[query.i];
        } else if (query.t == 1) {
            answers[query.index] = d;
        } else {
            answers[query.index] = s;
        }
    }

    for (ll x: answers) cout << x << '\n';
}

signed main() {
    cin.tie(NULL);
    ios_base::sync_with_stdio(false);

    #ifdef local
    freopen("input.txt", "r", stdin);
    #endif // local

    solve();
}
