#define _CRT_SECURE_NO_WARNINGS
#include <iostream>
#include <vector>
#include <algorithm>
#include <map>
#include <set>
#include <cassert>
#include <numeric>
#include <queue>

using namespace std;

typedef long long ll;
#define all(x) x.begin(), x.end()
#define rall(x) x.rbegin(), x.rend()

struct dsu {
	int n;
	vector<int> p, rang;
	dsu(int _n, vector<int>& rangs) {
		n = _n;
		p.resize(n);
		rang = rangs;
		iota(all(p), 0);
	}
	int lead(int v) {
		if (p[v] == v) return v;
		return p[v] = lead(p[v]);
	}
	int unite(int a, int b) {
		a = lead(a);
		b = lead(b);
		if (a == b) return 0;
		p[a] = b;
		return 1;
	}
};

#ifdef _DEBUG
const int MAXN = 1e3;
#else
const int MAXN = 12e5;
#endif

struct segtree {
	struct node {
		int val = 0, cnt = 0, s = 0;
		int dec = 0;
		node() {}
		node(int val, int cnt) : val(val* cnt), cnt(cnt), s(val) {
			if (val == 0) s = 0;
		}
	};
	int n;
	node tree[MAXN];
	void push(int v, int tl, int tr) {
		tree[v].val -= tree[v].dec * tree[v].s;
		tree[v].cnt -= tree[v].dec;
		if (tl != tr) {
			tree[v * 2 + 1].dec += tree[v].dec;
			tree[v * 2 + 2].dec += tree[v].dec;
		}
		tree[v].dec = 0;
	}
	node recalc(const node& l, const node& r) {
		node res;
		res.val = l.val + r.val;
		res.s = l.s + r.s;
		res.cnt = l.cnt + r.cnt;
		return res;
	}
	node recalc(int l, int r) {
		return recalc(tree[l], tree[r]);
	}
	void pull(int v, int tl, int tr) {
		push(v, tl, tr);
		tree[v] = recalc(v * 2 + 1, v * 2 + 2);
	}
	void build(int v, int tl, int tr, vector<int>& vals, int n) {
		if (tl == tr) {
			tree[v] = { vals[tl], n - tl - 1 };
			return;
		}
		int tm = (tl + tr) / 2;
		build(v * 2 + 1, tl, tm, vals, n);
		build(v * 2 + 2, tm + 1, tr, vals, n);
		pull(v, tl, tr);
	}
	segtree(vector<int>& vals) {
		n = vals.size();
		build(0, 0, n - 1, vals, n);
	}
	void update(int v, int tl, int tr, int L, int R, int dec) {
		push(v, tl, tr);
		if (R < tl || tr < L) return;
		if (L <= tl && tr <= R) {
			tree[v].dec += dec;
			push(v, tl, tr);
			return;
		}
		int tm = (tl + tr) / 2;
		update(v * 2 + 1, tl, tm, L, R, dec);
		update(v * 2 + 2, tm + 1, tr, L, R, dec);
		pull(v, tl, tr);
	}
	void update(int L, int R, int dec) {
		update(0, 0, n - 1, L, R, dec);
	}
	void update(int v, int tl, int tr, int x, int dec) {
		push(v, tl, tr);
		if (x < tl || tr < x) return;
		if (x <= tl && tr <= x) {
			tree[v].s += dec;
			tree[v].val = tree[v].s * tree[v].cnt;
			return;
		}
		int tm = (tl + tr) / 2;
		update(v * 2 + 1, tl, tm, x, dec);
		update(v * 2 + 2, tm + 1, tr, x, dec);
		pull(v, tl, tr);
	}
	void update(int x, int dec) {
		update(0, 0, n - 1, x, dec);
	}
	void off(int v, int tl, int tr, int x) {
		push(v, tl, tr);
		if (x < tl || tr < x) return;
		if (x <= tl && tr <= x) {
			tree[v].dec = tree[v].cnt;
			push(v, tl, tr);
			tree[v].s = 0;
			return;
		}
		int tm = (tl + tr) / 2;
		off(v * 2 + 1, tl, tm, x);
		off(v * 2 + 2, tm + 1, tr, x);
		pull(v, tl, tr);
	}
	void off(int x) {
		off(0, 0, n - 1, x);
	}
	node get(int v, int tl, int tr, int x) {
		push(v, tl, tr);
		if (x < tl || tr < x) return {};
		if (x <= tl && tr <= x) {
			return tree[v];
		}
		int tm = (tl + tr) / 2;
		return recalc(get(v * 2 + 1, tl, tm, x),
			get(v * 2 + 2, tm + 1, tr, x));
	}
	node get(int x) {
		return get(0, 0, n - 1, x);
	}
};

void off(int x, dsu& ds, segtree& tree) {
	tree.update(ds.lead(x + 1), tree.get(x).s);
	tree.off(x);
	tree.update(0, x - 1, 1);
}

signed main() {
#ifdef _DEBUG
	freopen("input.txt", "r", stdin);
	freopen("output.txt", "w", stdout);
#else
	cin.tie(0);
	ios_base::sync_with_stdio(false);
#endif
	int n, q;
	cin >> n >> q;
	vector<ll> c(n);
	map<ll, int> cnt;
	for (ll& i : c) {
		cin >> i;
		cnt[i]++;
	}
	sort(all(c));
	c.resize(unique(all(c)) - c.begin());
	vector<ll> r;
	for (int i = 1; i < c.size(); ++i)
		r.emplace_back(c[i] - c[i - 1]);

	vector<int> ccnt(cnt.size());
	int in = 0;
	for (auto& it : cnt)
		ccnt[in++] = it.second;
	segtree tree(ccnt);

	vector<pair<ll, int>> ss;
	for (int i = 0; i < r.size(); ++i) {
		ss.emplace_back(r[i], i);
	}
	sort(all(ss));
	dsu ds(r.size() + 1, ccnt);
	vector<pair<ll, ll>> aa(r.size() + 1);
	ll sum = 0, val = tree.tree[0].val;
	aa[0] = { sum, val };
	ll prt = 0;
	for (int i = 0; i < ss.size(); ++i) {
		sum += aa[i].second * (ss[i].first - prt);
		off(ds.lead(ss[i].second), ds, tree);
		ds.unite(ds.lead(ss[i].second), ds.lead(ss[i].second + 1));
		aa[i + 1] = { sum, tree.tree[0].val };
		prt = ss[i].first;
	}

	sort(all(r));
	ll t, k;
	for (int i = 0; i < q; ++i) {
		cin >> t >> k;
		if (t == 1) {
			auto it = upper_bound(all(r), k);
			cout << r.size() - (it - r.begin()) + 1 << '\n';
		}
		if (t == 2) {
			auto it = upper_bound(all(r), k);
			int in = (it - r.begin());
			if (in == 0)
				cout << aa[in].first + aa[in].second * k << '\n';
			else
				cout << aa[in].first + aa[in].second * (k - r[in - 1]) << '\n';
		}
	}
	return 0;
}