#ifdef LOCAL
#define _GLIBCXX_DEBUG
#else
// #pragma GCC optimize("O3")
// #pragma GCC optimize("fast-math")
// #pragma GCC optimize("unroll-loops")
// #pragma GCC optimize("Ofast")
// #pragma GCC target("avx2,popcnt,lzcnt,bmi,bmi2")
#endif

#include <algorithm>
#include <cassert>
#include <chrono>
#include <cstdlib>
#include <cstring>
#include <deque>
#include <iomanip>
#include <iostream>
#include <map>
#include <numeric>
#include <queue>
#include <random>
#include <set>
#include <unordered_map>
#include <unordered_set>
#include <vector>

using namespace std;
using ll = long long;

const ll INFLL = 0x3f3f3f3f3f3f3f3fll;

struct DSU {
    int n;
    vector<int> p, siz, cnt;

    DSU(int n) : n(n), p(n), siz(n, 1), cnt(n) { iota(p.begin(), p.end(), 0); }

    int find(int v) {
        if (p[v] == v) return v;
        return p[v] = find(p[v]);
    }

    bool merge(int v, int u) {
        v = find(v), u = find(u);
        if (v == u) return false;
        siz[v] += siz[u];
        cnt[v] += cnt[u];
        p[u] = v;
        return true;
    }
};

struct BIT {
    int n;
    vector<ll> t;

    BIT(int n) : n(n), t(n + 1, 0) {}

    static int f(int x) { return x & -x; }

    ll get(int r) {
        ll res = 0;
        for (; r > 0; r -= f(r)) res += t[r];
        return res;
    }

    void update(int x, ll dx) {
        for (x++; x <= n; x += f(x)) t[x] += dx;
    }
};

struct Event {
    int t;
    ll x;
    int i;
};

struct Query {
    int t;
    ll x;
    int i;
    ll ans = -1;
};

bool operator<(const Event &a, const Event &b) {
    return tie(a.x, a.t, a.i) < tie(b.x, b.t, b.i);
}

void solve() {
    int n, q;
    cin >> n >> q;
    vector<ll> c(n);
    map<ll, int> cnt;
    for (int i = 0; i < n; i++) {
        cin >> c[i];
        cnt[c[i]]++;
    }
    int m = (int)cnt.size();
    vector<pair<ll, int>> cntv(cnt.begin(), cnt.end());
    DSU dsu(m);
    int curcnt = m;
    vector<Query> qq(q);
    for (auto &[t, x, i, ans] : qq) {
        cin >> t >> x;
        if (t == 3) {
            cin >> i;
            i--;
        }
    }
    vector<Event> e;
    e.reserve(m + q);
    for (int i = 0; i < q; i++) {
        e.push_back({1, qq[i].x, i});
    }
    for (int i = 0; i < m - 1; i++) {
        e.push_back({0, cntv[i + 1].first - cntv[i].first, i});
    }
    sort(e.begin(), e.end());
    ll cursm = 0, smv = 0, last = 0;
    ll curval = 0;
    BIT smbit(m), fbit(m), valbit(m);
    for (int i = 0; i < m; i++) {
        smv += 1ll * cntv[i].second * (m - 1 - i);
        smbit.update(i, cntv[i].second);
        fbit.update(i, 1);
    }
    for (auto [t, x, i] : e) {
        if (t == 0) {
            cursm += smv * (x - last);
            last = x;
            curval += x;
            int v = dsu.find(i), u = dsu.find(i + 1);
            int cnt = dsu.cnt[u];
            smv -= smbit.get(u);
            smbit.update(u, -cnt);
            smbit.update(v, cnt);
            fbit.update(u, -1);
            valbit.update(u, x);
            if (dsu.merge(i, i + 1)) {
                curcnt--;
            }
        } else {
            if (qq[i].t == 1) qq[i].ans = curcnt;
            if (qq[i].t == 2) qq[i].ans = cursm + smv * (x - last);
            if (qq[i].t == 3) {
                ll y = c[qq[i].i];
                auto it = lower_bound(cntv.begin(), cntv.end(), pair{y, 0});
                int j = int(it - cntv.begin());
                // j = dsu.find(j);
                ll v = curcnt - fbit.get(j + 1);
                qq[i].ans = y + curval - valbit.get(j + 1) + v * x;
            }
        }
    }
    for (int i = 0; i < q; i++) {
        cout << qq[i].ans << '\n';
    }
}

signed main() {
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#endif

    int t = 1;
    // cin >> t;
    while (t--) solve();
}