#include <vector>
#include <iomanip>
#include <iostream>
#include <algorithm>

using namespace std;

const long long BASE = 1e18;
const int BASE_SZ = 18;

struct BigInt{
  bool sign;
  pair<long long, long long> val;
  BigInt() {
    val = make_pair(0, 0);
    sign = true;
  }
  BigInt(long long x) {
    if (x < 0) {
      sign = false;
      x *= -1;
    } else {
      sign = true;
    }
    val = make_pair(x % BASE, x / BASE);
  }
  BigInt(const pair<long long, long long> &_val, bool _sign) {
    val = _val;
    sign = _sign;
  }
};

void add(const pair<long long, long long> &a, const pair<long long, long long> &b, pair<long long, long long> &ans) {
  ans = make_pair(a.first + b.first, a.second + b.second);
  if (ans.first >= BASE) {
    ++ans.second;
    ans.first -= BASE;
  }
}

void sub(const pair<long long, long long> &a, const pair<long long, long long> &b, pair<long long, long long> &ans) {
  ans = make_pair(a.first - b.first, a.second - b.second);
  if (ans.first < 0) {
    ans.first += BASE;
    --ans.second;
  }
}

bool le(const pair<long long, long long> &a, const pair<long long, long long> &b) {
  if (a.second < b.second)
    return true;
  if (a.second > b.second)
    return false;
  return a.first < b.first;
}

void add(const BigInt &a, const BigInt &b, BigInt &c) {
  if (a.sign == b.sign) {
    add(a.val, b.val, c.val);
    c.sign = a.sign;
    return;
  }
  if (le(a.val, b.val)) {
    sub(b.val, a.val, c.val);
    c.sign = b.sign;
  } else {
    sub(a.val, b.val, c.val);
    c.sign = a.sign;
  }
}

bool operator <(const BigInt &a, const BigInt &b) {
  if (a.sign && !b.sign)
    return false;
  if (!a.sign && b.sign)
    return true;
  if (a.sign)
    return le(a.val, b.val);
  return le(b.val, a.val);
}

void ten_power(int k, int c, BigInt &to) {
  long long pw = c;
  for (int i = 0; i < k % BASE_SZ; ++i)
    pw *= 10;
  if (k < BASE_SZ)
    to.val = make_pair(pw, 0);
  else
    to.val = make_pair(0, pw);
}

const int K = 22;
BigInt dp[(1 << K) + 228];
int pr[(1 << K) + 228];

int main() {
#ifdef DEBUG
  freopen("input.txt", "r", stdin);
  freopen("output.txt", "w", stdout);
#else
  ios::sync_with_stdio(false);
  cin.tie(0);
  cout.tie(0);
#endif

  string s;
  cin >> s;
  long long y;
  cin >> y;
  int n = s.size();
  vector<int> A(n);
  int j = max(0, n - K);
  for (int i = 0; i < n; ++i)
    A[i] = s[i] - '0';
  sort(A.begin(), A.begin() + j, [](int a, int b) {
    return a > b;
  });
  for (int _ = 0; _ < K; ++_) {
    for (int l = j; l < n; ++l) {
      if (j && A[l] > A[j - 1]) {
        for (int t = l; t > j; --t)
          swap(A[t], A[t - 1]);
        for (int t = j; t > 0; --t) {
          if (A[t] > A[t - 1])
            swap(A[t], A[t - 1]);
        }
      }
    }
  }
  dp[0] = 0;
  BigInt tmp1, tmp2;
  for (int mask = 1; mask < (1 << (n - j)); ++mask) {
    ten_power(2 * BASE_SZ - 1, 1, dp[mask]);
    dp[mask].sign = false;
    int sz = __builtin_popcount(mask);
    int swap_cnt = 0;
    for (int i = 0; i < (n - j); ++i) {
      if (!(mask & (1 << i))) {
        ++swap_cnt;
        continue;
      }
      ten_power(n - j - sz, A[j + i], tmp2);
      add(dp[mask ^ (1 << i)], tmp2, tmp1);
      add(tmp1, BigInt(-y * swap_cnt), tmp2);
      if (dp[mask] < tmp2) {
        dp[mask] = tmp2;
        pr[mask] = i;
      }
    }
  }
  vector<int> B = A;
  int cur = (1 << (n - j)) - 1;
  while (cur) {
    int id = pr[cur];
    cur = cur ^ (1 << id);
    B[j + __builtin_popcount(cur)] = A[j + id];
  }
  for (int x : B)
    cout << x;
  cout << '\n';
  return 0;
}