#include <set>
#include <vector>
#include <iomanip>
#include <iostream>
#include <algorithm>

using namespace std;

const int MAXN = 200'200;
int par[MAXN];

int get(int v) {
  if (v == par[v])
    return v;
  return par[v] = get(par[v]);
}

void merge(int v, int u) {
  v = get(v);
  u = get(u);
  par[v] = u;
}

long long solve(const vector<long long> &A, const vector<long long> &C, const vector<pair<int, int>> &segms) {
  int n = A.size();
  int m = C.size();
  vector<int> order(m);
  for (int i = 0; i < m; ++i)
    order[i] = i;
  sort(order.begin(), order.end(), [&](int a, int b) {
    if (segms[a].second == segms[b].second)
      return segms[a].first < segms[b].first;
    return segms[a].second < segms[b].second;
  });
  vector<long long> A1 = A;
  for (int i = 0; i <= n; ++i)
    par[i] = i;
  long long ans = 0;
  for (int j : order) {
    long long rest = C[j];
    int cur = get(segms[j].first);
    while (cur <= segms[j].second) {
      if (A1[cur] <= rest) {
        ans += A1[cur];
        rest -= A1[cur];
        A1[cur] = 0;
        merge(cur, cur + 1);
        cur = get(cur);
      } else {
        ans += rest;
        A1[cur] -= rest;
        rest = 0;
        break;
      }
    }
  }
  return ans;
}

int main() {
#ifdef DEBUG
  freopen("input.txt", "r", stdin);
  freopen("output.txt", "w", stdout);
#else
  ios::sync_with_stdio(false);
  cin.tie(0);
  cout.tie(0);
#endif

  int q;
  cin >> q;
  while (q--) {
    int n, m;
    cin >> n >> m;
    vector<long long> A(n);
    for (int i = 0; i < n; ++i)
      cin >> A[i];
    vector<long long> C(m);
    vector<int> t(m);
    vector<pair<int, int>> segms(m);
    for (int i = 0; i < m; ++i) {
      cin >> segms[i].first >> segms[i].second >> C[i] >> t[i];
      --segms[i].first;
      --segms[i].second;
    }
    for (int x = 0; x < n; ++x) {
      vector<pair<int, int>> cur_segms(m);
      for (int i = 0; i < m; ++i) {
        if (t[i])
          cur_segms[i] = make_pair(min(segms[i].first, x), max(segms[i].second, x));
        else
          cur_segms[i] = segms[i];
      }
      cout << solve(A, C, cur_segms) << ' ';
    }
    cout << '\n';
  }
  return 0;
}
